import inspect
import os
import shlex
import subprocess
import sys
import unittest

sys.path.insert(0, "..")

from androidtv import constants

from .generate_test_constants import get_cmds


class TestConstants(unittest.TestCase):
    def setUp(self):
        self._cmds = []
        self._cmd_names = {}

    def assertCommand(self, value, expected_value):
        self.assertEqual(value, expected_value)
        self._cmds.append(self._cmd_names[value])

    @staticmethod
    def _exec(command):
        process = os.popen(command)
        output = process.read().strip()
        process.close()
        return output

    @staticmethod
    def _parse_current_app(dumpsys_output):
        return TestConstants._exec(
            'CURRENT_APP="' + dumpsys_output + '" && ' + constants.CMD_PARSE_CURRENT_APP + " && echo $CURRENT_APP"
        )

    @staticmethod
    def _hdmi_input(dumpsys_output):
        return TestConstants._exec(
            constants.CMD_HDMI_INPUT.replace("dumpsys activity starter", 'echo "' + dumpsys_output + '"')
        )

    def test_apps(self):
        """Test that the values (i.e., app names) in the ``APPS`` dict are unique."""
        apps_reversed = {val: key for key, val in constants.APPS.items()}
        self.assertEqual(len(constants.APPS), len(apps_reversed))

    def test_constants(self):
        """Test ADB shell commands.

        The contents of this test can be generated via the script tests/generate_test_constants.py.

        This is basically a form of version control for constants.

        """
        self.maxDiff = None

        # Generate a dictionary where the keys are the `CMD_*` strings and the values are their variable names
        cmds = get_cmds()
        self._cmd_names = {val: key for key, val in cmds.items()}

        # Check that each `CMD_*` is unique
        self.assertEqual(len(cmds), len(self._cmd_names))

        # CMD_AUDIO_STATE
        self.assertCommand(
            constants.CMD_AUDIO_STATE,
            r"dumpsys audio | grep paused | grep -qv 'Buffer Queue' && echo -e '1\c' || (dumpsys audio | grep started | grep -qv 'Buffer Queue' && echo '2\c' || echo '0\c')",
        )

        # CMD_AUDIO_STATE11
        self.assertCommand(
            constants.CMD_AUDIO_STATE11,
            r"CURRENT_AUDIO_STATE=$(dumpsys audio | sed -r -n '/[0-9]{2}-[0-9]{2}.*player piid:.*state:.*$/h; ${x;p;}') && echo $CURRENT_AUDIO_STATE | grep -q paused && echo -e '1\c' || { echo $CURRENT_AUDIO_STATE | grep -q started && echo '2\c' || echo '0\c' ; }",
        )

        # CMD_AWAKE
        self.assertCommand(constants.CMD_AWAKE, r"dumpsys power | grep mWakefulness | grep -q Awake")

        # CMD_CURRENT_APP
        self.assertCommand(
            constants.CMD_CURRENT_APP,
            r"CURRENT_APP=$(dumpsys window windows | grep -E 'mCurrentFocus|mFocusedApp') && CURRENT_APP=${CURRENT_APP#*ActivityRecord{* * } && CURRENT_APP=${CURRENT_APP#*{* * } && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP%\}*} && echo $CURRENT_APP",
        )

        # CMD_CURRENT_APP11
        self.assertCommand(
            constants.CMD_CURRENT_APP11,
            r"CURRENT_APP=$(dumpsys window windows | grep -E -m 1 'mInputMethod(Input)?Target') && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP##* } && echo $CURRENT_APP",
        )

        # CMD_CURRENT_APP12
        self.assertCommand(
            constants.CMD_CURRENT_APP12,
            r"CURRENT_APP=$(dumpsys window windows | grep -E 'mCurrentFocus|mFocusedApp|mObscuringWindow') && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP##* } && echo $CURRENT_APP",
        )

        # CMD_CURRENT_APP13
        self.assertCommand(
            constants.CMD_CURRENT_APP13,
            r"CURRENT_APP=$(dumpsys window windows | grep -E -m 1 'imeLayeringTarget|imeInputTarget|imeControlTarget') && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP##* } && echo $CURRENT_APP",
        )

        # CMD_CURRENT_APP_GOOGLE_TV
        self.assertCommand(
            constants.CMD_CURRENT_APP_GOOGLE_TV,
            r"CURRENT_APP=$(dumpsys activity a . | grep mResumedActivity) && CURRENT_APP=${CURRENT_APP#*ActivityRecord{* * } && CURRENT_APP=${CURRENT_APP#*{* * } && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP%\}*} && echo $CURRENT_APP",
        )

        # CMD_CURRENT_APP_MEDIA_SESSION_STATE
        self.assertCommand(
            constants.CMD_CURRENT_APP_MEDIA_SESSION_STATE,
            r"CURRENT_APP=$(dumpsys window windows | grep -E 'mCurrentFocus|mFocusedApp') && CURRENT_APP=${CURRENT_APP#*ActivityRecord{* * } && CURRENT_APP=${CURRENT_APP#*{* * } && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP%\}*} && echo $CURRENT_APP && dumpsys media_session | grep -A 100 'Sessions Stack' | grep -A 100 $CURRENT_APP | grep -m 1 'state=PlaybackState {'",
        )

        # CMD_CURRENT_APP_MEDIA_SESSION_STATE11
        self.assertCommand(
            constants.CMD_CURRENT_APP_MEDIA_SESSION_STATE11,
            r"CURRENT_APP=$(dumpsys window windows | grep -E -m 1 'mInputMethod(Input)?Target') && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP##* } && echo $CURRENT_APP && dumpsys media_session | grep -A 100 'Sessions Stack' | grep -A 100 $CURRENT_APP | grep -m 1 'state=PlaybackState {'",
        )

        # CMD_CURRENT_APP_MEDIA_SESSION_STATE12
        self.assertCommand(
            constants.CMD_CURRENT_APP_MEDIA_SESSION_STATE12,
            r"CURRENT_APP=$(dumpsys window windows | grep -E 'mCurrentFocus|mFocusedApp|mObscuringWindow') && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP##* } && echo $CURRENT_APP && dumpsys media_session | grep -A 100 'Sessions Stack' | grep -A 100 $CURRENT_APP | grep -m 1 'state=PlaybackState {'",
        )

        # CMD_CURRENT_APP_MEDIA_SESSION_STATE13
        self.assertCommand(
            constants.CMD_CURRENT_APP_MEDIA_SESSION_STATE13,
            r"CURRENT_APP=$(dumpsys window windows | grep -E -m 1 'imeLayeringTarget|imeInputTarget|imeControlTarget') && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP##* } && echo $CURRENT_APP && dumpsys media_session | grep -A 100 'Sessions Stack' | grep -A 100 $CURRENT_APP | grep -m 1 'state=PlaybackState {'",
        )

        # CMD_CURRENT_APP_MEDIA_SESSION_STATE_GOOGLE_TV
        self.assertCommand(
            constants.CMD_CURRENT_APP_MEDIA_SESSION_STATE_GOOGLE_TV,
            r"CURRENT_APP=$(dumpsys activity a . | grep mResumedActivity) && CURRENT_APP=${CURRENT_APP#*ActivityRecord{* * } && CURRENT_APP=${CURRENT_APP#*{* * } && CURRENT_APP=${CURRENT_APP%%/*} && CURRENT_APP=${CURRENT_APP%\}*} && echo $CURRENT_APP && dumpsys media_session | grep -A 100 'Sessions Stack' | grep -A 100 $CURRENT_APP | grep -m 1 'state=PlaybackState {'",
        )

        # CMD_DEVICE_PROPERTIES
        self.assertCommand(
            constants.CMD_DEVICE_PROPERTIES,
            r"getprop ro.product.manufacturer && getprop ro.product.model && getprop ro.serialno && getprop ro.build.version.release",
        )

        # CMD_HDMI_INPUT
        self.assertCommand(
            constants.CMD_HDMI_INPUT,
            r"dumpsys activity starter | grep -E -o '(ExternalTv|HDMI)InputService/HW[0-9]' -m 1 | grep -o 'HW[0-9]'",
        )

        # CMD_HDMI_INPUT11
        self.assertCommand(
            constants.CMD_HDMI_INPUT11,
            r"(HDMI=$(dumpsys tv_input | grep 'ResourceClientProfile {.*}' | grep -o -E '(hdmi_port=[0-9]|TV)') && { echo ${HDMI/hdmi_port=/HW} | cut -d' ' -f1 ; }) || dumpsys activity starter | grep -E -o '(ExternalTv|HDMI)InputService/HW[0-9]' -m 1 | grep -o 'HW[0-9]'",
        )

        # CMD_INSTALLED_APPS
        self.assertCommand(constants.CMD_INSTALLED_APPS, r"pm list packages")

        # CMD_LAUNCH_APP
        self.assertCommand(
            constants.CMD_LAUNCH_APP,
            r"CURRENT_APP=$(dumpsys window windows | grep -E 'mCurrentFocus|mFocusedApp') && CURRENT_APP=${{CURRENT_APP#*ActivityRecord{{* * }} && CURRENT_APP=${{CURRENT_APP#*{{* * }} && CURRENT_APP=${{CURRENT_APP%%/*}} && CURRENT_APP=${{CURRENT_APP%\}}*}} && if [ $CURRENT_APP != '{0}' ]; then monkey -p {0} -c android.intent.category.LEANBACK_LAUNCHER --pct-syskeys 0 1; fi",
        )

        # CMD_LAUNCH_APP11
        self.assertCommand(
            constants.CMD_LAUNCH_APP11,
            r"CURRENT_APP=$(dumpsys window windows | grep -E -m 1 'mInputMethod(Input)?Target') && CURRENT_APP=${{CURRENT_APP%%/*}} && CURRENT_APP=${{CURRENT_APP##* }} && if [ $CURRENT_APP != '{0}' ]; then monkey -p {0} -c android.intent.category.LEANBACK_LAUNCHER --pct-syskeys 0 1; fi",
        )

        # CMD_LAUNCH_APP12
        self.assertCommand(
            constants.CMD_LAUNCH_APP12,
            r"CURRENT_APP=$(dumpsys window windows | grep -E 'mCurrentFocus|mFocusedApp|mObscuringWindow') && CURRENT_APP=${{CURRENT_APP%%/*}} && CURRENT_APP=${{CURRENT_APP##* }} && if [ $CURRENT_APP != '{0}' ]; then monkey -p {0} -c android.intent.category.LEANBACK_LAUNCHER --pct-syskeys 0 1; fi",
        )

        # CMD_LAUNCH_APP13
        self.assertCommand(
            constants.CMD_LAUNCH_APP13,
            r"CURRENT_APP=$(dumpsys window windows | grep -E -m 1 'imeLayeringTarget|imeInputTarget|imeControlTarget') && CURRENT_APP=${{CURRENT_APP%%/*}} && CURRENT_APP=${{CURRENT_APP##* }} && if [ $CURRENT_APP != '{0}' ]; then monkey -p {0} -c android.intent.category.LEANBACK_LAUNCHER --pct-syskeys 0 1; fi",
        )

        # CMD_LAUNCH_APP_FIRETV
        self.assertCommand(
            constants.CMD_LAUNCH_APP_FIRETV,
            r"CURRENT_APP=$(dumpsys window windows | grep -E 'mCurrentFocus|mFocusedApp') && CURRENT_APP=${{CURRENT_APP#*ActivityRecord{{* * }} && CURRENT_APP=${{CURRENT_APP#*{{* * }} && CURRENT_APP=${{CURRENT_APP%%/*}} && CURRENT_APP=${{CURRENT_APP%\}}*}} && if [ $CURRENT_APP != '{0}' ]; then monkey -p {0} -c android.intent.category.LAUNCHER --pct-syskeys 0 1; fi",
        )

        # CMD_LAUNCH_APP_GOOGLE_TV
        self.assertCommand(
            constants.CMD_LAUNCH_APP_GOOGLE_TV,
            r"CURRENT_APP=$(dumpsys activity a . | grep mResumedActivity) && CURRENT_APP=${{CURRENT_APP#*ActivityRecord{{* * }} && CURRENT_APP=${{CURRENT_APP#*{{* * }} && CURRENT_APP=${{CURRENT_APP%%/*}} && CURRENT_APP=${{CURRENT_APP%\}}*}} && if [ $CURRENT_APP != '{0}' ]; then monkey -p {0} -c android.intent.category.LEANBACK_LAUNCHER --pct-syskeys 0 1; fi",
        )

        # CMD_MAC_ETH0
        self.assertCommand(constants.CMD_MAC_ETH0, r"ip addr show eth0 | grep -m 1 ether")

        # CMD_MAC_WLAN0
        self.assertCommand(constants.CMD_MAC_WLAN0, r"ip addr show wlan0 | grep -m 1 ether")

        # CMD_MANUFACTURER
        self.assertCommand(constants.CMD_MANUFACTURER, r"getprop ro.product.manufacturer")

        # CMD_MEDIA_SESSION_STATE
        self.assertCommand(
            constants.CMD_MEDIA_SESSION_STATE,
            r"dumpsys media_session | grep -A 100 'Sessions Stack' | grep -A 100 $CURRENT_APP | grep -m 1 'state=PlaybackState {'",
        )

        # CMD_MODEL
        self.assertCommand(constants.CMD_MODEL, r"getprop ro.product.model")

        # CMD_RUNNING_APPS_ANDROIDTV
        self.assertCommand(constants.CMD_RUNNING_APPS_ANDROIDTV, r"ps -A | grep u0_a")

        # CMD_RUNNING_APPS_FIRETV
        self.assertCommand(constants.CMD_RUNNING_APPS_FIRETV, r"ps | grep u0_a")

        # CMD_SCREEN_ON
        self.assertCommand(
            constants.CMD_SCREEN_ON,
            r"(dumpsys power | grep 'Display Power' | grep -q 'state=ON' || dumpsys power | grep -q 'mScreenOn=true' || dumpsys display | grep -q 'mScreenState=ON')",
        )

        # CMD_SCREEN_ON_AWAKE_WAKE_LOCK_SIZE
        self.assertCommand(
            constants.CMD_SCREEN_ON_AWAKE_WAKE_LOCK_SIZE,
            r"(dumpsys power | grep 'Display Power' | grep -q 'state=ON' || dumpsys power | grep -q 'mScreenOn=true' || dumpsys display | grep -q 'mScreenState=ON') && echo -e '1\c' || echo -e '0\c' && dumpsys power | grep mWakefulness | grep -q Awake && echo -e '1\c' || echo -e '0\c' && dumpsys power | grep Locks | grep 'size='",
        )

        # CMD_SERIALNO
        self.assertCommand(constants.CMD_SERIALNO, r"getprop ro.serialno")

        # CMD_STREAM_MUSIC
        self.assertCommand(constants.CMD_STREAM_MUSIC, r"dumpsys audio | grep '\- STREAM_MUSIC:' -A 11")

        # CMD_TURN_OFF_ANDROIDTV
        self.assertCommand(
            constants.CMD_TURN_OFF_ANDROIDTV,
            r"(dumpsys power | grep 'Display Power' | grep -q 'state=ON' || dumpsys power | grep -q 'mScreenOn=true' || dumpsys display | grep -q 'mScreenState=ON') && input keyevent 26",
        )

        # CMD_TURN_OFF_FIRETV
        self.assertCommand(
            constants.CMD_TURN_OFF_FIRETV,
            r"(dumpsys power | grep 'Display Power' | grep -q 'state=ON' || dumpsys power | grep -q 'mScreenOn=true' || dumpsys display | grep -q 'mScreenState=ON') && input keyevent 223",
        )

        # CMD_TURN_ON_ANDROIDTV
        self.assertCommand(
            constants.CMD_TURN_ON_ANDROIDTV,
            r"(dumpsys power | grep 'Display Power' | grep -q 'state=ON' || dumpsys power | grep -q 'mScreenOn=true' || dumpsys display | grep -q 'mScreenState=ON') || input keyevent 26",
        )

        # CMD_TURN_ON_FIRETV
        self.assertCommand(
            constants.CMD_TURN_ON_FIRETV,
            r"(dumpsys power | grep 'Display Power' | grep -q 'state=ON' || dumpsys power | grep -q 'mScreenOn=true' || dumpsys display | grep -q 'mScreenState=ON') || (input keyevent 26 && input keyevent 3)",
        )

        # CMD_VERSION
        self.assertCommand(constants.CMD_VERSION, r"getprop ro.build.version.release")

        # CMD_VOLUME_SET_COMMAND
        self.assertCommand(constants.CMD_VOLUME_SET_COMMAND, r"media volume --show --stream 3 --set {}")

        # CMD_VOLUME_SET_COMMAND11
        self.assertCommand(
            constants.CMD_VOLUME_SET_COMMAND11,
            r"cmd media_session volume --show --stream 3 --set {}",
        )

        # CMD_WAKE_LOCK_SIZE
        self.assertCommand(constants.CMD_WAKE_LOCK_SIZE, r"dumpsys power | grep Locks | grep 'size='")

        # Assert that the keys were checked in alphabetical order
        self.assertEqual(self._cmds, sorted(cmds.keys()))

    @unittest.skipIf(sys.version_info.major == 2, "Test requires Python 3")
    def test_no_underscores(self):
        """Test that 'ANDROID_TV', 'BASE_TV', and 'FIRE_TV' do not appear in the code base."""
        cwd = os.path.join(os.path.dirname(__file__), "..")
        for underscore_name in ["ANDROID_TV", "BASE_TV", "FIRE_TV"]:
            with subprocess.Popen(
                shlex.split("git grep -l {} -- androidtv/".format(underscore_name)),
                cwd=cwd,
            ) as p:
                self.assertEqual(p.wait(), 1)

    def test_current_app_extraction_atv_launcher(self):
        dumpsys_output = """
            mCurrentFocus=Window{e74bb23 u0 com.google.android.tvlauncher/com.google.android.tvlauncher.MainActivity}
            mFocusedApp=AppWindowToken{c791eb8 token=Token{a6bad1b ActivityRecord{35dbb2a u0 com.google.android.tvlauncher/.MainActivity t2719}}}
        """
        current_app = self._parse_current_app(dumpsys_output)

        self.assertEqual(current_app, constants.APP_ATV_LAUNCHER)

    def test_current_app_extraction_atv_launcher_google_tv(self):
        dumpsys_output = """
            mResumedActivity: ActivityRecord{35dbb2a u0 com.google.android.tvlauncher/.MainActivity t2719}
        """
        current_app = self._parse_current_app(dumpsys_output)

        self.assertEqual(current_app, constants.APP_ATV_LAUNCHER)

    def test_current_app_extraction_sony_picture_off(self):
        dumpsys_output = """
            mCurrentFocus=Window{c52eaa8 u0 com.sony.dtv.sonysystemservice}
            mFocusedApp=AppWindowToken{11da138 token=Token{19ea99b ActivityRecord{c4d9aa u0 tunein.player/tunein.ui.leanback.ui.activities.TvHomeActivity t2424}}}
        """
        current_app = self._parse_current_app(dumpsys_output)

        self.assertEqual(current_app, constants.APP_TUNEIN)

    def test_current_app_extraction_intent_with_no_activity(self):
        dumpsys_output = """
            mCurrentFocus=Window{c52eaa8 u0 com.sony.dtv.sonysystemservice}
        """
        current_app = self._parse_current_app(dumpsys_output)

        self.assertEqual(current_app, "com.sony.dtv.sonysystemservice")

    def test_current_app_extraction_with_identifier_as_output(self):
        dumpsys_output = constants.APP_NETFLIX
        current_app = self._parse_current_app(dumpsys_output)

        self.assertEqual(current_app, constants.APP_NETFLIX)

    def test_hdmi_input(self):
        dumpsys_output = """
             Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.mediatek.tvinput%2F.hdmi.HDMIInputService%2FHW5 flg=0x10000000 cmp=org.droidtv.playtv/.PlayTvActivity (has extras) }
            mIntent=Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.mediatek.tvinput/.hdmi.HDMIInputService/HW5 flg=0x10000000 cmp=org.droidtv.playtv/.PlayTvActivity (has extras) }
        """
        hdmi_input = self._hdmi_input(dumpsys_output)

        self.assertEqual(hdmi_input, "HW5")

    def test_hdmi_input_sony(self):
        dumpsys_output = """
            ACTIVITY MANAGER ACTIVITIES (dumpsys activity starter)
            ActivityStarter:
              mCurrentUser=0
              mLastStartReason=startActivityAsUser
              mLastStartActivityTimeMs=4 Feb 2021 12:30:29
              mLastStartActivityResult=2
              mLastStartActivityRecord:
               packageName=com.sony.dtv.tvx processName=com.sony.dtv.tvx
               launchedFromUid=10134 launchedFromPackage=com.sony.dtv.tvx userId=0
               app=ProcessRecord{edc6562 3904:com.sony.dtv.tvx/u0a134}
               Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.sony.dtv.tvinput.external%2F.ExternalTvInputService%2FHW2 flg=0x10040000 cmp=com.sony.dtv.tvx/.MainActivity (has extras) }
               frontOfTask=true task=TaskRecord{9eecc8c #2830 A=com.sony.dtv.tvx U=0 StackId=1 sz=1}
               taskAffinity=com.sony.dtv.tvx
               realActivity=com.sony.dtv.tvx/.MainActivity
               baseDir=/system/priv-app/Tvx/Tvx.apk
               dataDir=/data/user/0/com.sony.dtv.tvx
               stateNotNeeded=false componentSpecified=false mActivityType=0
               compat={320dpi always-compat} labelRes=0x7f090048 icon=0x7f02006d theme=0x7f0b01c2
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               taskDescription: iconFilename=null label="null" primaryColor=ff8c0000
                backgroundColor=ff303030
                statusBarColor=ff000000
                navigationBarColor=ff000000
               launchFailed=false launchCount=0 lastLaunchTime=-1h46m13s561ms
               haveState=false icicle=null
               state=RESUMED stopped=false delayedResume=false finishing=false
               keysPaused=false inHistory=true visible=true sleeping=false idle=true mStartingWindowState=STARTING_WINDOW_REMOVED
               fullscreen=true noDisplay=false immersive=false launchMode=3
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=APPLICATION_ACTIVITY_TYPE
               waitingVisible=false nowVisible=true lastVisibleTime=-8s281ms
               connections=[ConnectionRecord{c68246 u0 CR com.sony.dtv.osdplanevisibilitymanager/.OsdEnabler:@cf54e21}]
               resizeMode=RESIZE_MODE_RESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
               supportsPictureInPicture=true
               supportsPictureInPictureWhilePausing: true
              mLastHomeActivityStartResult=0
              mLastHomeActivityStartRecord:
               packageName=com.google.android.tvlauncher processName=com.google.android.tvlauncher
               launchedFromUid=0 launchedFromPackage=null userId=0
               app=ProcessRecord{b28e70e 4013:com.google.android.tvlauncher/u0a162}
               Intent { act=android.intent.action.MAIN cat=[android.intent.category.HOME] flg=0x10800100 cmp=com.google.android.tvlauncher/.MainActivity }
               frontOfTask=true task=TaskRecord{1142677 #2719 A=.TvLauncher U=0 StackId=0 sz=1}
               taskAffinity=.TvLauncher
               realActivity=com.google.android.tvlauncher/.MainActivity
               baseDir=/data/app/com.google.android.tvlauncher-yGwF-XP6Zf5hV0DbQ9Z4gA==/base.apk
               dataDir=/data/user/0/com.google.android.tvlauncher
               stateNotNeeded=true componentSpecified=false mActivityType=1
               compat={320dpi always-compat} labelRes=0x7f12002b icon=0x7f0f0000 theme=0x7f130007
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               taskDescription: iconFilename=null label="null" primaryColor=ff37474f
                backgroundColor=ff303030
                statusBarColor=ff263238
                navigationBarColor=ff000000
               launchFailed=false launchCount=0 lastLaunchTime=-11h32m28s448ms
               haveState=true icicle=Bundle[mParcelledData.dataSize=1272]
               state=STOPPED stopped=true delayedResume=false finishing=false
               keysPaused=false inHistory=true visible=false sleeping=false idle=true mStartingWindowState=STARTING_WINDOW_REMOVED
               fullscreen=true noDisplay=false immersive=false launchMode=2
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=HOME_ACTIVITY_TYPE
               waitingVisible=false nowVisible=false lastVisibleTime=-16s965ms
               resizeMode=RESIZE_MODE_UNRESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
              mStartActivity:
               packageName=com.sony.dtv.tvx processName=com.sony.dtv.tvx
               launchedFromUid=10134 launchedFromPackage=com.sony.dtv.tvx userId=0
               app=null
               Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.sony.dtv.tvinput.external%2F.ExternalTvInputService%2FHW2 flg=0x10440000 cmp=com.sony.dtv.tvx/.MainActivity (has extras) }
               frontOfTask=false task=TaskRecord{9eecc8c #2830 A=com.sony.dtv.tvx U=0 StackId=1 sz=1}
               taskAffinity=com.sony.dtv.tvx
               realActivity=com.sony.dtv.tvx/.MainActivity
               baseDir=/system/priv-app/Tvx/Tvx.apk
               dataDir=/data/user/0/com.sony.dtv.tvx
               stateNotNeeded=false componentSpecified=false mActivityType=0
               compat=null labelRes=0x7f090048 icon=0x7f02006d theme=0x7f0b01c2
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={0.0 ?mcc?mnc ?localeList ?layoutDir ?swdp ?wdp ?hdp ?density ?lsize ?long ?ldr ?wideColorGamut ?orien ?uimode ?night ?touch ?keyb/?/? ?nav/?}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               launchFailed=false launchCount=0 lastLaunchTime=0
               haveState=true icicle=null
               state=INITIALIZING stopped=false delayedResume=false finishing=false
               keysPaused=false inHistory=false visible=false sleeping=false idle=false mStartingWindowState=STARTING_WINDOW_NOT_SHOWN
               fullscreen=true noDisplay=false immersive=false launchMode=3
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=APPLICATION_ACTIVITY_TYPE
               resizeMode=RESIZE_MODE_RESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
               supportsPictureInPicture=true
               supportsPictureInPictureWhilePausing: false
              mIntent=Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.sony.dtv.tvinput.external/.ExternalTvInputService/HW2 flg=0x10440000 cmp=com.sony.dtv.tvx/.MainActivity (has extras) }
              mLaunchSingleTop=false mLaunchSingleInstance=true mLaunchSingleTask=false mLaunchFlags=0x10040000 mDoResume=true mAddingToTask=false
        """
        hdmi_input = self._hdmi_input(dumpsys_output)

        self.assertEqual(hdmi_input, "HW2")

    def test_hdmi_input_sony_netflix(self):
        dumpsys_output = """
            ACTIVITY MANAGER ACTIVITIES (dumpsys activity starter)
            ActivityStarter:
              mCurrentUser=0
              mLastStartReason=startActivityAsUser
              mLastStartActivityTimeMs=6 Feb 2021 22:34:59
              mLastStartActivityResult=2
              mLastStartActivityRecord:
               packageName=com.netflix.ninja processName=com.netflix.ninja
               launchedFromUid=10162 launchedFromPackage=com.google.android.tvlauncher userId=0
               app=ProcessRecord{8005dce 16003:com.netflix.ninja/u0a143}
               Intent { act=android.intent.action.VIEW dat=http://www.netflix.com/home flg=0x10000020 pkg=com.netflix.ninja cmp=com.netflix.ninja/.MainActivity bnds=[140,476][300,636] (has extras) }
               frontOfTask=true task=TaskRecord{5c24cd8 #2727 A=com.netflix.ninja U=0 StackId=1 sz=1}
               taskAffinity=com.netflix.ninja
               realActivity=com.netflix.ninja/.MainActivity
               baseDir=/data/app/com.netflix.ninja-BRRRO8oABNAvcXQxHb4oBA==/base.apk
               dataDir=/data/user/0/com.netflix.ninja
               stateNotNeeded=false componentSpecified=true mActivityType=0
               compat={320dpi always-compat} labelRes=0x7f0e001c icon=0x7f070175 theme=0x7f0f0139
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               taskDescription: iconFilename=null label="null" primaryColor=ff000000
                backgroundColor=ff000000
                statusBarColor=ff000000
                navigationBarColor=ff000000
               launchFailed=false launchCount=0 lastLaunchTime=-1d2h24m34s18ms
               haveState=false icicle=null
               state=RESUMED stopped=false delayedResume=false finishing=false
               keysPaused=false inHistory=true visible=true sleeping=false idle=true mStartingWindowState=STARTING_WINDOW_REMOVED
               fullscreen=true noDisplay=false immersive=false launchMode=2
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=APPLICATION_ACTIVITY_TYPE
               waitingVisible=false nowVisible=true lastVisibleTime=-25s492ms
               connections=[ConnectionRecord{64e94 u0 CR com.netflix.ninja/.NetflixService:@a0a82e7}, ConnectionRecord{91d022a u0 com.google.android.apps.mediashell/.MediaShellCastReceiverService:@2400f15}]
               resizeMode=RESIZE_MODE_UNRESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
              mLastHomeActivityStartResult=0
              mLastHomeActivityStartRecord:
               packageName=com.google.android.tvlauncher processName=com.google.android.tvlauncher
               launchedFromUid=0 launchedFromPackage=null userId=0
               app=ProcessRecord{8d5fa46 22638:com.google.android.tvlauncher/u0a162}
               Intent { act=android.intent.action.MAIN cat=[android.intent.category.HOME] flg=0x10800100 cmp=com.google.android.tvlauncher/.MainActivity }
               frontOfTask=true task=TaskRecord{1142677 #2719 A=.TvLauncher U=0 StackId=0 sz=1}
               taskAffinity=.TvLauncher
               realActivity=com.google.android.tvlauncher/.MainActivity
               baseDir=/data/app/com.google.android.tvlauncher-yGwF-XP6Zf5hV0DbQ9Z4gA==/base.apk
               dataDir=/data/user/0/com.google.android.tvlauncher
               stateNotNeeded=true componentSpecified=false mActivityType=1
               compat={320dpi always-compat} labelRes=0x7f12002b icon=0x7f0f0000 theme=0x7f130007
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               taskDescription: iconFilename=null label="null" primaryColor=ff37474f
                backgroundColor=ff303030
                statusBarColor=ff263238
                navigationBarColor=ff000000
               launchFailed=false launchCount=0 lastLaunchTime=-4m48s633ms
               haveState=true icicle=Bundle[mParcelledData.dataSize=1216]
               state=STOPPED stopped=true delayedResume=false finishing=false
               keysPaused=false inHistory=true visible=false sleeping=false idle=true mStartingWindowState=STARTING_WINDOW_REMOVED
               fullscreen=true noDisplay=false immersive=false launchMode=2
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=HOME_ACTIVITY_TYPE
               waitingVisible=false nowVisible=false lastVisibleTime=-4m46s108ms
               resizeMode=RESIZE_MODE_UNRESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
              mStartActivity:
               packageName=com.netflix.ninja processName=com.netflix.ninja
               launchedFromUid=10143 launchedFromPackage=com.netflix.ninja userId=0
               app=null
               Intent { flg=0x10420000 cmp=com.netflix.ninja/.MainActivity (has extras) }
               frontOfTask=false task=TaskRecord{5c24cd8 #2727 A=com.netflix.ninja U=0 StackId=1 sz=1}
               taskAffinity=com.netflix.ninja
               realActivity=com.netflix.ninja/.MainActivity
               baseDir=/data/app/com.netflix.ninja-BRRRO8oABNAvcXQxHb4oBA==/base.apk
               dataDir=/data/user/0/com.netflix.ninja
               stateNotNeeded=false componentSpecified=true mActivityType=0
               compat=null labelRes=0x7f0e001c icon=0x7f070175 theme=0x7f0f0139
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={0.0 ?mcc?mnc ?localeList ?layoutDir ?swdp ?wdp ?hdp ?density ?lsize ?long ?ldr ?wideColorGamut ?orien ?uimode ?night ?touch ?keyb/?/? ?nav/?}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               launchFailed=false launchCount=0 lastLaunchTime=0
               haveState=true icicle=null
               state=INITIALIZING stopped=false delayedResume=false finishing=false
               keysPaused=false inHistory=false visible=false sleeping=false idle=false mStartingWindowState=STARTING_WINDOW_NOT_SHOWN
               fullscreen=true noDisplay=false immersive=false launchMode=2
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=APPLICATION_ACTIVITY_TYPE
               resizeMode=RESIZE_MODE_UNRESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
              mIntent=Intent { flg=0x10420000 cmp=com.netflix.ninja/.MainActivity (has extras) }
              mLaunchSingleTop=false mLaunchSingleInstance=false mLaunchSingleTask=true mLaunchFlags=0x10020000 mDoResume=true mAddingToTask=false
        """
        hdmi_input = self._hdmi_input(dumpsys_output)

        self.assertEqual(hdmi_input, "")

    def test_hdmi_input_sony_non_hardware_definition(self):
        dumpsys_output = """
            ACTIVITY MANAGER ACTIVITIES (dumpsys activity starter)
            ActivityStarter:
              mCurrentUser=0
              mLastStartReason=startActivityAsUser
              mLastStartActivityTimeMs=6 Feb 2021 22:30:46
              mLastStartActivityResult=2
              mLastStartActivityRecord:
               packageName=com.sony.dtv.tvx processName=com.sony.dtv.tvx
               launchedFromUid=10134 launchedFromPackage=com.sony.dtv.tvx userId=0
               app=ProcessRecord{edc6562 3904:com.sony.dtv.tvx/u0a134}
               Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.sony.dtv.tvinput.external%2F.ExternalTvInputService%2FHW2 flg=0x10040000 cmp=com.sony.dtv.tvx/.MainActivity (has extras) }
               frontOfTask=true task=TaskRecord{9eecc8c #2830 A=com.sony.dtv.tvx U=0 StackId=1 sz=1}
               taskAffinity=com.sony.dtv.tvx
               realActivity=com.sony.dtv.tvx/.MainActivity
               baseDir=/system/priv-app/Tvx/Tvx.apk
               dataDir=/data/user/0/com.sony.dtv.tvx
               stateNotNeeded=false componentSpecified=false mActivityType=0
               compat={320dpi always-compat} labelRes=0x7f090048 icon=0x7f02006d theme=0x7f0b01c2
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               taskDescription: iconFilename=null label="null" primaryColor=ff8c0000
                backgroundColor=ff303030
                statusBarColor=ff000000
                navigationBarColor=ff000000
               launchFailed=false launchCount=0 lastLaunchTime=-2d11h49m7s584ms
               haveState=false icicle=null
               state=RESUMED stopped=false delayedResume=false finishing=false
               keysPaused=false inHistory=true visible=true sleeping=false idle=true mStartingWindowState=STARTING_WINDOW_REMOVED
               fullscreen=true noDisplay=false immersive=false launchMode=3
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=APPLICATION_ACTIVITY_TYPE
               waitingVisible=false nowVisible=true lastVisibleTime=-2m50s467ms
               connections=[ConnectionRecord{fee4c3e u0 CR com.sony.dtv.osdplanevisibilitymanager/.OsdEnabler:@246a4f9}]
               resizeMode=RESIZE_MODE_RESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
               supportsPictureInPicture=true
               supportsPictureInPictureWhilePausing: true
              mLastHomeActivityStartResult=0
              mLastHomeActivityStartRecord:
               packageName=com.google.android.tvlauncher processName=com.google.android.tvlauncher
               launchedFromUid=0 launchedFromPackage=null userId=0
               app=ProcessRecord{8d5fa46 22638:com.google.android.tvlauncher/u0a162}
               Intent { act=android.intent.action.MAIN cat=[android.intent.category.HOME] flg=0x10800100 cmp=com.google.android.tvlauncher/.MainActivity }
               frontOfTask=true task=TaskRecord{1142677 #2719 A=.TvLauncher U=0 StackId=0 sz=1}
               taskAffinity=.TvLauncher
               realActivity=com.google.android.tvlauncher/.MainActivity
               baseDir=/data/app/com.google.android.tvlauncher-yGwF-XP6Zf5hV0DbQ9Z4gA==/base.apk
               dataDir=/data/user/0/com.google.android.tvlauncher
               stateNotNeeded=true componentSpecified=false mActivityType=1
               compat={320dpi always-compat} labelRes=0x7f12002b icon=0x7f0f0000 theme=0x7f130007
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               taskDescription: iconFilename=null label="null" primaryColor=ff37474f
                backgroundColor=ff303030
                statusBarColor=ff263238
                navigationBarColor=ff000000
               launchFailed=false launchCount=0 lastLaunchTime=-3m0s967ms
               haveState=true icicle=Bundle[mParcelledData.dataSize=1216]
               state=STOPPED stopped=true delayedResume=false finishing=false
               keysPaused=false inHistory=true visible=false sleeping=false idle=true mStartingWindowState=STARTING_WINDOW_SHOWN
               fullscreen=true noDisplay=false immersive=false launchMode=2
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=HOME_ACTIVITY_TYPE
               waitingVisible=false nowVisible=false lastVisibleTime=-2m58s442ms
               resizeMode=RESIZE_MODE_UNRESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
              mStartActivity:
               packageName=com.sony.dtv.tvx processName=com.sony.dtv.tvx
               launchedFromUid=10134 launchedFromPackage=com.sony.dtv.tvx userId=0
               app=null
               Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.sony.dtv.tvinput.external%2F.ExternalTvInputService%2FHDMI100004 flg=0x10400000 cmp=com.sony.dtv.tvx/.MainActivity (has extras) }
               frontOfTask=false task=TaskRecord{9eecc8c #2830 A=com.sony.dtv.tvx U=0 StackId=1 sz=1}
               taskAffinity=com.sony.dtv.tvx
               realActivity=com.sony.dtv.tvx/.MainActivity
               baseDir=/system/priv-app/Tvx/Tvx.apk
               dataDir=/data/user/0/com.sony.dtv.tvx
               stateNotNeeded=false componentSpecified=false mActivityType=0
               compat=null labelRes=0x7f090048 icon=0x7f02006d theme=0x7f0b01c2
               mLastReportedConfigurations:
                mGlobalConfig={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
                mOverrideConfig={0.0 ?mcc?mnc ?localeList ?layoutDir ?swdp ?wdp ?hdp ?density ?lsize ?long ?ldr ?wideColorGamut ?orien ?uimode ?night ?touch ?keyb/?/? ?nav/?}
               CurrentConfiguration={1.0 ?mcc?mnc [en_GB] ldltr sw540dp w960dp h540dp 320dpi lrg long hdr land television -touch -keyb/v/h dpad/v appBounds=Rect(0, 0 - 1920, 1080) s.3}
               launchFailed=false launchCount=0 lastLaunchTime=0
               haveState=true icicle=null
               state=INITIALIZING stopped=false delayedResume=false finishing=false
               keysPaused=false inHistory=false visible=false sleeping=false idle=false mStartingWindowState=STARTING_WINDOW_NOT_SHOWN
               fullscreen=true noDisplay=false immersive=false launchMode=3
               frozenBeforeDestroy=false forceNewConfig=false
               mActivityType=APPLICATION_ACTIVITY_TYPE
               resizeMode=RESIZE_MODE_RESIZEABLE
               mLastReportedMultiWindowMode=false mLastReportedPictureInPictureMode=false
               supportsPictureInPicture=true
               supportsPictureInPictureWhilePausing: false
              mIntent=Intent { act=android.intent.action.VIEW dat=content://android.media.tv/passthrough/com.sony.dtv.tvinput.external/.ExternalTvInputService/HDMI100004 flg=0x10400000 cmp=com.sony.dtv.tvx/.MainActivity (has extras) }
              mLaunchSingleTop=false mLaunchSingleInstance=true mLaunchSingleTask=false mLaunchFlags=0x10000000 mDoResume=true mAddingToTask=false
        """
        hdmi_input = self._hdmi_input(dumpsys_output)

        self.assertEqual(hdmi_input, "")

    def test_apps_alphabetized(self):
        constants_str = inspect.getsource(constants)
        last_app = ""
        for line in constants_str.splitlines():
            if line.startswith("APP_"):
                app = line.split()[0]
                self.assertEqual(app, app.upper())
                self.assertEqual(sorted([last_app, app])[0], last_app)
                last_app = app

        apps_dict_lines = []
        for line in constants_str.splitlines():
            if line.startswith("APPS = {"):
                apps_dict_lines.append(line[len("APPS = {") :].strip())
            elif apps_dict_lines:
                apps_dict_lines.append(line.strip())
                if line.strip().endswith("}"):
                    break

        apps_dict_lines_sorted = sorted(apps_dict_lines)
        self.assertListEqual(apps_dict_lines, apps_dict_lines_sorted)


if __name__ == "__main__":
    unittest.main()



from contextlib import contextmanager
import sys
import unittest

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

sys.path.insert(0, "..")

from adb_shell.transport.tcp_transport import TcpTransport
from androidtv.adb_manager.adb_manager_sync import _acquire, ADBPythonSync, ADBServerSync
from androidtv.exceptions import LockNotAcquiredException
from . import patchers


if sys.version_info[0] == 2:
    FileNotFoundError = IOError


class Read(object):
    """Mock an opened file that can be read."""

    def read(self):
        return ""


class ReadFail(object):
    """Mock an opened file that cannot be read."""

    def read(self):
        raise FileNotFoundError


PNG_IMAGE = b"\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\n\x00\x00\x00\n\x08\x06\x00\x00\x00\x8d2\xcf\xbd\x00\x00\x00\x04sBIT\x08\x08\x08\x08|\x08d\x88\x00\x00\x00\tpHYs\x00\x00\x0fa\x00\x00\x0fa\x01\xa8?\xa7i\x00\x00\x00\x0eIDAT\x18\x95c`\x18\x05\x83\x13\x00\x00\x01\x9a\x00\x01\x16\xca\xd3i\x00\x00\x00\x00IEND\xaeB`\x82"

PNG_IMAGE_NEEDS_REPLACING = PNG_IMAGE[:5] + b"\r" + PNG_IMAGE[5:]


@contextmanager
def open_priv(infile):
    """A patch that will read the private key but not the public key."""
    try:
        if infile == "adbkey":
            yield Read()
        else:
            yield ReadFail()
    finally:
        pass


@contextmanager
def open_priv_pub(infile):
    try:
        yield Read()
    finally:
        pass


class FakeLock(object):
    def __init__(self, *args, **kwargs):
        self._acquired = True

    def acquire(self, *args, **kwargs):
        if self._acquired:
            self._acquired = False
            return True
        return self._acquired

    def release(self, *args, **kwargs):
        self._acquired = True


class LockedLock(FakeLock):
    def __init__(self, *args, **kwargs):
        self._acquired = False


class TestADBPythonSync(unittest.TestCase):
    """Test the `ADBPythonSync` class."""

    PATCH_KEY = "python"

    def setUp(self):
        """Create an `ADBPythonSync` instance."""
        with patchers.PATCH_ADB_DEVICE_TCP, patchers.patch_connect(True)[self.PATCH_KEY]:
            self.adb = ADBPythonSync("HOST", 5555)

    def test_locked_lock(self):
        """Test that the ``FakeLock`` class works as expected."""
        with patch.object(self.adb, "_adb_lock", FakeLock()):
            with _acquire(self.adb._adb_lock):
                with self.assertRaises(LockNotAcquiredException):
                    with _acquire(self.adb._adb_lock):
                        pass

            with _acquire(self.adb._adb_lock) as acquired:
                self.assertTrue(acquired)

    def test_connect_success(self):
        """Test when the connect attempt is successful."""
        with patchers.patch_connect(True)[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())
            self.assertTrue(self.adb.available)

    def test_connect_fail(self):
        """Test when the connect attempt fails."""
        with patchers.patch_connect(False)[self.PATCH_KEY]:
            self.assertFalse(self.adb.connect())
            self.assertFalse(self.adb.available)

        with patchers.patch_connect(True)[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())
            self.assertTrue(self.adb.available)

        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patchers.PATCH_CONNECT_FAIL_CUSTOM_EXCEPTION[self.PATCH_KEY]:
                self.assertFalse(self.adb.connect())
                self.assertFalse(self.adb.available)

    def test_connect_fail_lock(self):
        """Test when the connect attempt fails due to the lock."""
        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patch.object(self.adb, "_adb_lock", LockedLock()):
                self.assertFalse(self.adb.connect())
                self.assertFalse(self.adb.available)

    def test_adb_shell_fail(self):
        """Test when an ADB shell command is not sent because the device is unavailable."""
        self.assertFalse(self.adb.available)
        with patchers.patch_connect(True)[self.PATCH_KEY], patchers.patch_shell(None)[self.PATCH_KEY]:
            self.assertIsNone(self.adb.shell("TEST"))

        with patchers.patch_connect(True)[self.PATCH_KEY], patchers.patch_shell("TEST")[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())
            with patch.object(self.adb, "_adb_lock", LockedLock()):
                with self.assertRaises(LockNotAcquiredException):
                    self.adb.shell("TEST")

                with self.assertRaises(LockNotAcquiredException):
                    self.adb.shell("TEST2")

    def test_adb_shell_success(self):
        """Test when an ADB shell command is successfully sent."""
        with patchers.patch_connect(True)[self.PATCH_KEY], patchers.patch_shell("TEST")[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())
            self.assertEqual(self.adb.shell("TEST"), "TEST")

    def test_adb_shell_fail_lock_released(self):
        """Test that the ADB lock gets released when an exception is raised."""
        with patchers.patch_connect(True)[self.PATCH_KEY], patchers.patch_shell("TEST")[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())

        with patchers.patch_shell("TEST", error=True)[self.PATCH_KEY], patch.object(self.adb, "_adb_lock", FakeLock()):
            with patch("{}.FakeLock.release".format(__name__)) as release:
                with self.assertRaises(Exception):
                    self.adb.shell("TEST")
                assert release.called

    def test_adb_shell_lock_not_acquired_not_released(self):
        """Test that the lock does not get released if it is not acquired."""
        with patchers.patch_connect(True)[self.PATCH_KEY], patchers.patch_shell("TEST")[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())
            self.assertEqual(self.adb.shell("TEST"), "TEST")

        with patchers.patch_shell("TEST")[self.PATCH_KEY], patch.object(self.adb, "_adb_lock", LockedLock()):
            with patch("{}.LockedLock.release".format(__name__)) as release:
                with self.assertRaises(LockNotAcquiredException):
                    self.adb.shell("TEST")

                release.assert_not_called()

    def test_adb_push_fail(self):
        """Test when an ADB push command is not executed because the device is unavailable."""
        self.assertFalse(self.adb.available)
        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patchers.PATCH_PUSH[self.PATCH_KEY] as patch_push:
                self.adb.push("TEST_LOCAL_PATCH", "TEST_DEVICE_PATH")
                patch_push.assert_not_called()

        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patchers.PATCH_PUSH[self.PATCH_KEY] as patch_push:
                self.assertTrue(self.adb.connect())
                with patch.object(self.adb, "_adb_lock", LockedLock()):
                    with self.assertRaises(LockNotAcquiredException):
                        self.adb.push("TEST_LOCAL_PATH", "TEST_DEVICE_PATH")

                    patch_push.assert_not_called()

    def test_adb_push_success(self):
        """Test when an ADB push command is successfully executed."""
        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patchers.PATCH_PUSH[self.PATCH_KEY] as patch_push:
                self.assertTrue(self.adb.connect())
                self.adb.push("TEST_LOCAL_PATH", "TEST_DEVICE_PATH")
                self.assertEqual(patch_push.call_count, 1)

    def test_adb_pull_fail(self):
        """Test when an ADB pull command is not executed because the device is unavailable."""
        self.assertFalse(self.adb.available)
        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patchers.PATCH_PULL[self.PATCH_KEY] as patch_pull:
                self.adb.pull("TEST_LOCAL_PATCH", "TEST_DEVICE_PATH")
                patch_pull.assert_not_called()

        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patchers.PATCH_PULL[self.PATCH_KEY] as patch_pull:
                self.assertTrue(self.adb.connect())
                with patch.object(self.adb, "_adb_lock", LockedLock()):
                    with self.assertRaises(LockNotAcquiredException):
                        self.adb.pull("TEST_LOCAL_PATH", "TEST_DEVICE_PATH")
                    patch_pull.assert_not_called()

    def test_adb_pull_success(self):
        """Test when an ADB pull command is successfully executed."""
        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patchers.PATCH_PULL[self.PATCH_KEY] as patch_pull:
                self.assertTrue(self.adb.connect())
                self.adb.pull("TEST_LOCAL_PATH", "TEST_DEVICE_PATH")
                self.assertEqual(patch_pull.call_count, 1)

    def test_adb_screencap_fail_unavailable(self):
        """Test when an ADB screencap command fails because the connection is unavailable."""
        self.assertFalse(self.adb.available)
        self.assertIsNone(self.adb.screencap())

    def test_adb_screencap_lock_not_acquired(self):
        """Test when an ADB screencap command fails because the ADB lock could not be acquired."""
        with patchers.patch_connect(True)[self.PATCH_KEY], patchers.patch_shell("TEST")[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())
            self.assertEqual(self.adb.shell("TEST"), "TEST")

        with patchers.patch_shell(PNG_IMAGE)[self.PATCH_KEY], patch.object(self.adb, "_adb_lock", LockedLock()):
            with patch("{}.LockedLock.release".format(__name__)) as release:
                with self.assertRaises(LockNotAcquiredException):
                    self.adb.screencap()

                release.assert_not_called()

    def test_adb_screencap_success(self):
        """Test the `screencap` method."""
        with patchers.patch_connect(True)[self.PATCH_KEY], patchers.patch_shell(PNG_IMAGE)[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())

            if isinstance(self.adb, ADBPythonSync):
                self.assertEqual(self.adb.screencap(), PNG_IMAGE)

                with patchers.patch_shell(PNG_IMAGE_NEEDS_REPLACING)[self.PATCH_KEY]:
                    self.assertEqual(self.adb.screencap(), PNG_IMAGE)

            else:
                with patch.object(self.adb._adb_device, "screencap", return_value=PNG_IMAGE):
                    self.assertEqual(self.adb.screencap(), PNG_IMAGE)


class TestADBPythonUsbSync(TestADBPythonSync):
    """Test the `ADBPythonSync` class using a USB connection."""

    def setUp(self):
        """Create an `ADBPythonSync` instance with a USB connection."""
        # Patch the real `AdbDeviceUsb` with the fake `AdbDeviceTcpFake`
        with patchers.PATCH_ADB_DEVICE_USB, patchers.patch_connect(True)[self.PATCH_KEY]:
            self.adb = ADBPythonSync("", 5555)


class TestADBServerSync(TestADBPythonSync):
    """Test the `ADBServerSync` class."""

    PATCH_KEY = "server"

    def setUp(self):
        """Create an `ADBServerSync` instance."""
        self.adb = ADBServerSync("HOST", 5555, "ADB_SERVER_IP")

    def test_connect_fail_server(self):
        """Test that the ``connect`` method works correctly."""
        with patchers.patch_connect(True)[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())

            with patchers.PATCH_ADB_SERVER_RUNTIME_ERROR:
                self.assertFalse(self.adb.connect())
                self.assertFalse(self.adb.available)


class TestADBPythonSyncWithAuthentication(unittest.TestCase):
    """Test the `ADBPythonSync` class."""

    PATCH_KEY = "python"

    def setUp(self):
        """Create an `ADBPythonSync` instance."""
        with patchers.PATCH_ADB_DEVICE_TCP, patchers.patch_connect(True)[self.PATCH_KEY]:
            self.adb = ADBPythonSync("HOST", 5555, "adbkey")

    def test_connect_success_with_priv_key(self):
        """Test when the connect attempt is successful when using a private key."""
        with patchers.patch_connect(True)[self.PATCH_KEY], patch(
            "androidtv.adb_manager.adb_manager_sync.open", open_priv
        ), patch("androidtv.adb_manager.adb_manager_sync.PythonRSASigner", return_value="TEST"):
            self.assertTrue(self.adb.connect())
            self.assertTrue(self.adb.available)

        with patchers.patch_connect(True)[self.PATCH_KEY]:
            with patch("androidtv.adb_manager.adb_manager_sync.open") as patch_open:
                self.assertTrue(self.adb.connect())
                self.assertTrue(self.adb.available)
                assert not patch_open.called

    def test_connect_success_with_priv_pub_key(self):
        """Test when the connect attempt is successful when using private and public keys."""
        with patchers.patch_connect(True)[self.PATCH_KEY], patch(
            "androidtv.adb_manager.adb_manager_sync.open", open_priv_pub
        ), patch("androidtv.adb_manager.adb_manager_sync.PythonRSASigner", return_value=None):
            self.assertTrue(self.adb.connect())
            self.assertTrue(self.adb.available)


class TestADBPythonSyncClose(unittest.TestCase):
    """Test the `ADBPythonSync.close` method."""

    PATCH_KEY = "python"

    def test_close(self):
        """Test the `ADBPythonSync.close` method."""
        with patchers.PATCH_ADB_DEVICE_TCP, patchers.patch_connect(True)[self.PATCH_KEY]:
            self.adb = ADBPythonSync("HOST", 5555)

        with patchers.patch_connect(True)[self.PATCH_KEY]:
            self.assertTrue(self.adb.connect())
            self.assertTrue(self.adb.available)

            self.adb.close()
            self.assertFalse(self.adb.available)



import sys
import unittest
from unittest.mock import patch


sys.path.insert(0, "..")

from androidtv import constants
from androidtv.firetv.firetv_async import FireTVAsync

from . import async_patchers
from .async_wrapper import awaiter
from .patchers import patch_calls


class TestFireTVAsyncPython(unittest.TestCase):
    ADB_ATTR = "_adb"
    PATCH_KEY = "python"

    @awaiter
    async def setUp(self):
        with async_patchers.PATCH_ADB_DEVICE_TCP, async_patchers.patch_connect(True)[
            self.PATCH_KEY
        ], async_patchers.patch_shell("")[self.PATCH_KEY]:
            self.ftv = FireTVAsync("HOST", 5555)
            await self.ftv.adb_connect()

    @awaiter
    async def test_turn_on_off(self):
        """Test that the ``FireTVAsync.turn_on`` and ``FireTVAsync.turn_off`` methods work correctly."""
        with async_patchers.patch_connect(True)[self.PATCH_KEY], async_patchers.patch_shell("")[self.PATCH_KEY]:
            await self.ftv.turn_on()
            self.assertEqual(
                getattr(self.ftv._adb, self.ADB_ATTR).shell_cmd,
                constants.CMD_SCREEN_ON
                + " || (input keyevent {0} && input keyevent {1})".format(constants.KEY_POWER, constants.KEY_HOME),
            )

            await self.ftv.turn_off()
            self.assertEqual(
                getattr(self.ftv._adb, self.ADB_ATTR).shell_cmd,
                constants.CMD_SCREEN_ON + " && input keyevent {0}".format(constants.KEY_SLEEP),
            )

    @awaiter
    async def test_send_intent(self):
        """Test that the ``_send_intent`` method works correctly."""
        with async_patchers.patch_shell("output\r\nretcode")[self.PATCH_KEY]:
            result = await self.ftv._send_intent("TEST", constants.INTENT_LAUNCH_FIRETV)
            self.assertEqual(
                getattr(self.ftv._adb, self.ADB_ATTR).shell_cmd,
                "monkey -p TEST -c android.intent.category.LAUNCHER 1; echo $?",
            )
            self.assertDictEqual(result, {"output": "output", "retcode": "retcode"})

        with async_patchers.patch_connect(True)[self.PATCH_KEY], async_patchers.patch_shell(None)[self.PATCH_KEY]:
            result = await self.ftv._send_intent("TEST", constants.INTENT_LAUNCH_FIRETV)
            self.assertEqual(
                getattr(self.ftv._adb, self.ADB_ATTR).shell_cmd,
                "monkey -p TEST -c android.intent.category.LAUNCHER 1; echo $?",
            )
            self.assertDictEqual(result, {})

    @awaiter
    async def test_launch_app_stop_app(self):
        """Test that the ``FireTVAsync.launch_app`` and ``FireTVAsync.stop_app`` methods work correctly."""
        with async_patchers.patch_shell("")[self.PATCH_KEY]:
            await self.ftv.launch_app("TEST")
            self.assertEqual(
                getattr(self.ftv._adb, self.ADB_ATTR).shell_cmd, constants.CMD_LAUNCH_APP_FIRETV.format("TEST")
            )

            await self.ftv.stop_app("TEST")
            self.assertEqual(getattr(self.ftv._adb, self.ADB_ATTR).shell_cmd, "am force-stop TEST")

    @awaiter
    async def test_running_apps(self):
        """Check that the ``running_apps`` property works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(self.ftv, self.ftv._running_apps) as patched:
                await self.ftv.running_apps()
                assert patched.called

    @awaiter
    async def test_get_properties(self):
        """Check that ``get_properties()`` works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(
                self.ftv, self.ftv.screen_on_awake_wake_lock_size
            ) as screen_on_awake_wake_lock_size, patch_calls(
                self.ftv, self.ftv.current_app_media_session_state
            ) as current_app_media_session_state, patch_calls(
                self.ftv, self.ftv.running_apps
            ) as running_apps, patch_calls(
                self.ftv, self.ftv.get_hdmi_input
            ) as get_hdmi_input:
                await self.ftv.get_properties(lazy=True)
                assert screen_on_awake_wake_lock_size.called
                assert not current_app_media_session_state.called
                assert not running_apps.called
                assert not get_hdmi_input.called

            with patch_calls(
                self.ftv, self.ftv.screen_on_awake_wake_lock_size
            ) as screen_on_awake_wake_lock_size, patch_calls(
                self.ftv, self.ftv.current_app_media_session_state
            ) as current_app_media_session_state, patch_calls(
                self.ftv, self.ftv.running_apps
            ) as running_apps, patch_calls(
                self.ftv, self.ftv.get_hdmi_input
            ) as get_hdmi_input:
                await self.ftv.get_properties(lazy=False, get_running_apps=True)
                assert screen_on_awake_wake_lock_size.called
                assert current_app_media_session_state.called
                assert running_apps.called
                assert get_hdmi_input.called

            with patch_calls(
                self.ftv, self.ftv.screen_on_awake_wake_lock_size
            ) as screen_on_awake_wake_lock_size, patch_calls(
                self.ftv, self.ftv.current_app_media_session_state
            ) as current_app_media_session_state, patch_calls(
                self.ftv, self.ftv.running_apps
            ) as running_apps, patch_calls(
                self.ftv, self.ftv.get_hdmi_input
            ) as get_hdmi_input:
                await self.ftv.get_properties(lazy=False, get_running_apps=False)
                assert screen_on_awake_wake_lock_size.called
                assert current_app_media_session_state.called
                assert not running_apps.called
                assert get_hdmi_input.called

    @awaiter
    async def test_get_properties_dict(self):
        """Check that ``get_properties_dict()`` works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(self.ftv, self.ftv.get_properties) as get_properties:
                await self.ftv.get_properties_dict()
                assert get_properties.called

    @awaiter
    async def test_update(self):
        """Check that the ``update`` method works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(self.ftv, self.ftv.get_properties) as patched:
                await self.ftv.update()
                assert patched.called


class TestFireTVAsyncServer(TestFireTVAsyncPython):
    ADB_ATTR = "_adb_device"
    PATCH_KEY = "server"

    @awaiter
    async def setUp(self):
        with async_patchers.patch_connect(True)[self.PATCH_KEY], async_patchers.patch_shell("")[self.PATCH_KEY]:
            self.ftv = FireTVAsync("HOST", 5555, adb_server_ip="ADB_SERVER_IP")
            await self.ftv.adb_connect()


if __name__ == "__main__":
    unittest.main()



"""Define patches used for androidtv tests."""

from unittest.mock import patch

try:
    from unittest.mock import AsyncMock
except ImportError:
    from unittest.mock import MagicMock

    class AsyncMock(MagicMock):
        async def __call__(self, *args, **kwargs):
            return super(AsyncMock, self).__call__(*args, **kwargs)


KEY_PYTHON = "python"
KEY_SERVER = "server"

ADB_DEVICE_TCP_ASYNC_FAKE = "AdbDeviceTcpAsyncFake"
CLIENT_ASYNC_FAKE_SUCCESS = "ClientAsyncFakeSuccess"
CLIENT_ASYNC_FAKE_FAIL = "ClientAsyncFakeFail"
DEVICE_ASYNC_FAKE = "DeviceAsyncFake"


def async_patch(*args, **kwargs):
    return patch(*args, new_callable=AsyncMock, **kwargs)


class AdbDeviceTcpAsyncFake(object):
    """A fake of the `adb_shell.adb_device_async.AdbDeviceTcpAsync` class."""

    def __init__(self, *args, **kwargs):
        """Initialize a fake `adb_shell.adb_device_async.AdbDeviceTcpAsync` instance."""
        self.available = False

    async def close(self):
        """Close the socket connection."""
        self.available = False

    async def connect(self, *args, **kwargs):
        """Try to connect to a device."""
        raise NotImplementedError

    async def push(self, *args, **kwargs):
        """Push a file to the device."""

    async def pull(self, *args, **kwargs):
        """Pull a file from the device."""

    async def shell(self, cmd, *args, **kwargs):
        """Send an ADB shell command."""
        return None


class ClientAsyncFakeSuccess(object):
    """A fake of the `ppadb.client.Client` class when the connection and shell commands succeed."""

    def __init__(self, host="127.0.0.1", port=5037):
        """Initialize a `ClientAsyncFakeSuccess` instance."""
        self._devices = []

    async def device(self, serial):
        """Mock the `ClientAsync.device` method when the device is connected via ADB."""
        device = DeviceAsyncFake(serial)
        self._devices.append(device)
        return device


class ClientAsyncFakeFail(object):
    """A fake of the `ppadb.client.Client` class when the connection and shell commands fail."""

    def __init__(self, host="127.0.0.1", port=5037):
        """Initialize a `ClientAsyncFakeFail` instance."""
        self._devices = []

    async def device(self, serial):
        """Mock the `ClientAsync.device` method when the device is not connected via ADB."""
        self._devices = []


class DeviceAsyncFake(object):
    """A fake of the `ppadb.device.Device` class."""

    def __init__(self, host):
        """Initialize a `DeviceAsyncFake` instance."""
        self.host = host

    async def push(self, *args, **kwargs):
        """Push a file to the device."""

    async def pull(self, *args, **kwargs):
        """Pull a file from the device."""

    async def shell(self, cmd):
        """Send an ADB shell command."""
        raise NotImplementedError

    async def screencap(self):
        """Take a screencap."""
        raise NotImplementedError


def patch_connect(success):
    """Mock the `adb_shell.adb_device_async.AdbDeviceTcpAsync` and `ppadb.client.Client` classes."""

    async def connect_success_python(self, *args, **kwargs):
        """Mock the `AdbDeviceTcpAsyncFake.connect` method when it succeeds."""
        self.available = True

    async def connect_fail_python(self, *args, **kwargs):
        """Mock the `AdbDeviceTcpAsyncFake.connect` method when it fails."""
        raise OSError

    if success:
        return {
            KEY_PYTHON: patch("{}.{}.connect".format(__name__, ADB_DEVICE_TCP_ASYNC_FAKE), connect_success_python),
            KEY_SERVER: patch("androidtv.adb_manager.adb_manager_async.ClientAsync", ClientAsyncFakeSuccess),
        }
    return {
        KEY_PYTHON: patch("{}.{}.connect".format(__name__, ADB_DEVICE_TCP_ASYNC_FAKE), connect_fail_python),
        KEY_SERVER: patch("androidtv.adb_manager.adb_manager_async.ClientAsync", ClientAsyncFakeFail),
    }


def patch_shell(response=None, error=False):
    """Mock the `AdbDeviceTcpAsyncFake.shell` and `DeviceAsyncFake.shell` methods."""

    if isinstance(response, list):
        response_list = response
    else:
        response_list = [response]

    next_response = 0

    async def shell_success(self, cmd, *args, **kwargs):
        """Mock the `AdbDeviceTcpAsyncFake.shell` and `DeviceAsyncFake.shell` methods when they are successful."""
        self.shell_cmd = cmd
        nonlocal next_response
        nonlocal response_list
        res = response_list[next_response]
        next_response = (next_response + 1) % len(response_list)
        return res

    async def shell_fail_python(self, cmd, *args, **kwargs):
        """Mock the `AdbDeviceTcpAsyncFake.shell` method when it fails."""
        self.shell_cmd = cmd
        raise AttributeError

    async def shell_fail_server(self, cmd):
        """Mock the `DeviceAsyncFake.shell` method when it fails."""
        self.shell_cmd = cmd
        raise ConnectionResetError

    if not error:
        return {
            KEY_PYTHON: patch("{}.{}.shell".format(__name__, ADB_DEVICE_TCP_ASYNC_FAKE), shell_success),
            KEY_SERVER: patch("{}.{}.shell".format(__name__, DEVICE_ASYNC_FAKE), shell_success),
        }
    return {
        KEY_PYTHON: patch("{}.{}.shell".format(__name__, ADB_DEVICE_TCP_ASYNC_FAKE), shell_fail_python),
        KEY_SERVER: patch("{}.{}.shell".format(__name__, DEVICE_ASYNC_FAKE), shell_fail_server),
    }


PATCH_PUSH = {
    KEY_PYTHON: async_patch("{}.{}.push".format(__name__, ADB_DEVICE_TCP_ASYNC_FAKE)),
    KEY_SERVER: async_patch("{}.{}.push".format(__name__, DEVICE_ASYNC_FAKE)),
}

PATCH_PULL = {
    KEY_PYTHON: async_patch("{}.{}.pull".format(__name__, ADB_DEVICE_TCP_ASYNC_FAKE)),
    KEY_SERVER: async_patch("{}.{}.pull".format(__name__, DEVICE_ASYNC_FAKE)),
}

PATCH_ADB_DEVICE_TCP = patch("androidtv.adb_manager.adb_manager_async.AdbDeviceTcpAsync", AdbDeviceTcpAsyncFake)

PATCH_ADB_SERVER_RUNTIME_ERROR = async_patch(
    "{}.{}.device".format(__name__, CLIENT_ASYNC_FAKE_SUCCESS), side_effect=RuntimeError
)


class CustomException(Exception):
    """A custom exception type."""


PATCH_CONNECT_FAIL_CUSTOM_EXCEPTION = {
    KEY_PYTHON: async_patch("{}.{}.connect".format(__name__, ADB_DEVICE_TCP_ASYNC_FAKE), side_effect=CustomException),
    KEY_SERVER: async_patch("{}.{}.device".format(__name__, CLIENT_ASYNC_FAKE_SUCCESS), side_effect=CustomException),
}



import asyncio
import sys
import unittest
from unittest.mock import patch


sys.path.insert(0, "..")

from androidtv import constants
from androidtv.androidtv.androidtv_async import AndroidTVAsync

from . import async_patchers
from .async_wrapper import awaiter
from .patchers import patch_calls


STREAM_MUSIC_EMPTY = "- STREAM_MUSIC:\n \n- STREAM"

STREAM_MUSIC_OFF = """- STREAM_MUSIC:
   Muted: false
   Min: 0
   Max: 60
   Current: 2 (speaker): 20, 40000 (hmdi_arc): 27, 40000000 (default): 15
   Devices: speaker
- STREAM_ALARM:
   Muted: true
   Min: 0
   Max: 7
   Current: 2 (speaker): 3, 40000 (hmdi_arc): 3, 40000000 (default): 2
   Devices: speaker"""

STREAM_MUSIC_NO_VOLUME = """- STREAM_MUSIC:
   Muted: false
   Min: 0
   Max: 60
   Devices: speaker
- STREAM_ALARM:
   Muted: true
   Min: 0
   Max: 7
   Current: 2 (speaker): 3, 40000 (hmdi_arc): 3, 40000000 (default): 2
   Devices: speaker"""


STREAM_MUSIC_ON = """- STREAM_MUSIC:
   Muted: false
   Min: 0
   Max: 60
   Current: 2 (speaker): 20, 40000 (hmdi_arc): 22, 40000000 (default): 15
   Devices: hmdi_arc
- STREAM_ALARM:
   Muted: false
   Min: 0
   Max: 7
   Current: 2 (speaker): 3, 40000 (hmdi_arc): 3, 40000000 (default): 2
   Devices: speaker"""


class TestAndroidTVAsyncPython(unittest.TestCase):
    PATCH_KEY = "python"
    ADB_ATTR = "_adb"

    @awaiter
    async def setUp(self):
        with async_patchers.PATCH_ADB_DEVICE_TCP, async_patchers.patch_connect(True)[
            self.PATCH_KEY
        ], async_patchers.patch_shell("")[self.PATCH_KEY]:
            self.atv = AndroidTVAsync("HOST", 5555)
            await self.atv.adb_connect()

    @awaiter
    async def test_turn_on_off(self):
        """Test that the ``AndroidTVAsync.turn_on`` and ``AndroidTVAsync.turn_off`` methods work correctly."""
        with async_patchers.patch_connect(True)[self.PATCH_KEY], async_patchers.patch_shell("")[self.PATCH_KEY]:
            await self.atv.turn_on()
            self.assertEqual(
                getattr(self.atv._adb, self.ADB_ATTR).shell_cmd,
                constants.CMD_SCREEN_ON + " || input keyevent {0}".format(constants.KEY_POWER),
            )

            await self.atv.turn_off()
            self.assertEqual(
                getattr(self.atv._adb, self.ADB_ATTR).shell_cmd,
                constants.CMD_SCREEN_ON + " && input keyevent {0}".format(constants.KEY_POWER),
            )

    @awaiter
    async def test_start_intent(self):
        """Test that the ``start_intent`` method works correctly."""
        with async_patchers.patch_connect(True)[self.PATCH_KEY], async_patchers.patch_shell("")[self.PATCH_KEY]:
            await self.atv.start_intent("TEST")
            self.assertEqual(
                getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "am start -a android.intent.action.VIEW -d TEST"
            )

    @awaiter
    async def test_running_apps(self):
        """Check that the ``running_apps`` property works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(self.atv, self.atv._running_apps) as patched:
                await self.atv.running_apps()
                assert patched.called

    @awaiter
    async def test_stream_music_properties(self):
        """Check that the ``stream_music_properties`` method works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(self.atv, self.atv._audio_output_device) as audio_output_device, patch_calls(
                self.atv, self.atv._is_volume_muted
            ) as is_volume_muted, patch_calls(self.atv, self.atv._volume) as volume, patch_calls(
                self.atv, self.atv._volume_level
            ) as volume_level:
                await self.atv.stream_music_properties()
                assert audio_output_device.called
                assert is_volume_muted.called
                assert volume.called
                assert volume_level.called

            with patch_calls(self.atv, self.atv._audio_output_device) as audio_output_device:
                await self.atv.audio_output_device()
                assert audio_output_device.called

            with patch_calls(self.atv, self.atv._is_volume_muted) as is_volume_muted:
                await self.atv.is_volume_muted()
                assert is_volume_muted.called

            with patch_calls(self.atv, self.atv._volume) as volume:
                await self.atv.volume()
                assert volume.called

            with patch_calls(self.atv, self.atv._volume_level) as volume_level:
                await self.atv.volume_level()
                assert volume_level.called

    @awaiter
    async def test_set_volume_level(self):
        """Check that the ``set_volume_level`` method works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            new_volume_level = await self.atv.set_volume_level(0.5)
            self.assertIsNone(new_volume_level)

        with async_patchers.patch_shell("")[self.PATCH_KEY]:
            new_volume_level = await self.atv.set_volume_level(0.5)
            self.assertIsNone(new_volume_level)

        with async_patchers.patch_shell(STREAM_MUSIC_ON)[self.PATCH_KEY]:
            new_volume_level = await self.atv.set_volume_level(0.5)
            self.assertEqual(new_volume_level, 0.5)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "media volume --show --stream 3 --set 30")

        with async_patchers.patch_shell("")[self.PATCH_KEY]:
            new_volume_level = await self.atv.set_volume_level(30.0 / 60)
            self.assertEqual(new_volume_level, 0.5)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "media volume --show --stream 3 --set 30")

        with async_patchers.patch_shell("")[self.PATCH_KEY]:
            new_volume_level = await self.atv.set_volume_level(22.0 / 60)
            self.assertEqual(new_volume_level, 22.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "media volume --show --stream 3 --set 22")

    @awaiter
    async def test_volume_up(self):
        """Check that the ``volume_up`` method works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_up()
            self.assertIsNone(new_volume_level)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 24")

        with async_patchers.patch_shell("")[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_up()
            self.assertIsNone(new_volume_level)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 24")

        with async_patchers.patch_shell(STREAM_MUSIC_ON)[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_up()
            self.assertEqual(new_volume_level, 23.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 24")
            new_volume_level = await self.atv.volume_up(23.0 / 60)
            self.assertEqual(new_volume_level, 24.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 24")

        with async_patchers.patch_shell(STREAM_MUSIC_OFF)[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_up()
            self.assertEqual(new_volume_level, 21.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 24")
            new_volume_level = await self.atv.volume_up(21.0 / 60)
            self.assertEqual(new_volume_level, 22.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 24")

    @awaiter
    async def test_volume_down(self):
        """Check that the ``volume_down`` method works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_down()
            self.assertIsNone(new_volume_level)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 25")

        with async_patchers.patch_shell("")[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_down()
            self.assertIsNone(new_volume_level)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 25")

        with async_patchers.patch_shell(STREAM_MUSIC_ON)[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_down()
            self.assertEqual(new_volume_level, 21.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 25")
            new_volume_level = await self.atv.volume_down(21.0 / 60)
            self.assertEqual(new_volume_level, 20.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 25")

        with async_patchers.patch_shell(STREAM_MUSIC_OFF)[self.PATCH_KEY]:
            new_volume_level = await self.atv.volume_down()
            self.assertEqual(new_volume_level, 19.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 25")
            new_volume_level = await self.atv.volume_down(19.0 / 60)
            self.assertEqual(new_volume_level, 18.0 / 60)
            self.assertEqual(getattr(self.atv._adb, self.ADB_ATTR).shell_cmd, "input keyevent 25")

    @awaiter
    async def test_get_properties(self):
        """Check that ``get_properties()`` works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(
                self.atv, self.atv.screen_on_awake_wake_lock_size
            ) as screen_on_awake_wake_lock_size, patch_calls(
                self.atv, self.atv.current_app_media_session_state
            ) as current_app_media_session_state, patch_calls(
                self.atv, self.atv.stream_music_properties
            ) as stream_music_properties, patch_calls(
                self.atv, self.atv.running_apps
            ) as running_apps, patch_calls(
                self.atv, self.atv.get_hdmi_input
            ) as get_hdmi_input:
                await self.atv.get_properties(lazy=True)
                assert screen_on_awake_wake_lock_size.called
                assert not current_app_media_session_state.called
                assert not running_apps.called
                assert not get_hdmi_input.called

            with patch_calls(
                self.atv, self.atv.screen_on_awake_wake_lock_size
            ) as screen_on_awake_wake_lock_size, patch_calls(
                self.atv, self.atv.current_app_media_session_state
            ) as current_app_media_session_state, patch_calls(
                self.atv, self.atv.stream_music_properties
            ) as stream_music_properties, patch_calls(
                self.atv, self.atv.running_apps
            ) as running_apps, patch_calls(
                self.atv, self.atv.get_hdmi_input
            ) as get_hdmi_input:
                await self.atv.get_properties(lazy=False, get_running_apps=True)
                assert screen_on_awake_wake_lock_size.called
                assert current_app_media_session_state.called
                assert running_apps.called
                assert get_hdmi_input.called

            with patch_calls(
                self.atv, self.atv.screen_on_awake_wake_lock_size
            ) as screen_on_awake_wake_lock_size, patch_calls(
                self.atv, self.atv.current_app_media_session_state
            ) as current_app_media_session_state, patch_calls(
                self.atv, self.atv.stream_music_properties
            ) as stream_music_properties, patch_calls(
                self.atv, self.atv.running_apps
            ) as running_apps, patch_calls(
                self.atv, self.atv.get_hdmi_input
            ) as get_hdmi_input:
                await self.atv.get_properties(lazy=False, get_running_apps=False)
                assert screen_on_awake_wake_lock_size.called
                assert current_app_media_session_state.called
                assert not running_apps.called
                assert get_hdmi_input.called

    @awaiter
    async def test_get_properties_dict(self):
        """Check that ``get_properties_dict()`` works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(self.atv, self.atv.get_properties) as get_properties:
                await self.atv.get_properties_dict()
                assert get_properties.called

    @awaiter
    async def test_update(self):
        """Check that the ``update`` method works correctly."""
        with async_patchers.patch_shell(None)[self.PATCH_KEY]:
            with patch_calls(self.atv, self.atv._update) as patched:
                await self.atv.update()
                assert patched.called


class TestAndroidTVAsyncServer(TestAndroidTVAsyncPython):
    PATCH_KEY = "server"
    ADB_ATTR = "_adb_device"

    @awaiter
    async def setUp(self):
        with async_patchers.patch_connect(True)[self.PATCH_KEY], async_patchers.patch_shell("")[self.PATCH_KEY]:
            self.atv = AndroidTVAsync("HOST", 5555, adb_server_ip="ADB_SERVER_IP")
            await self.atv.adb_connect()


if __name__ == "__main__":
    unittest.main()



"""setup.py file for the androidtv package."""

from setuptools import setup

with open("README.rst") as f:
    readme = f.read()

setup(
    name="androidtv",
    version="0.0.73",
    description="Communicate with an Android TV or Fire TV device via ADB over a network.",
    long_description=readme,
    keywords=["adb", "android", "androidtv", "firetv"],
    url="https://github.com/JeffLIrion/python-androidtv/",
    license="MIT",
    author="Jeff Irion",
    author_email="jefflirion@users.noreply.github.com",
    packages=["androidtv", "androidtv.adb_manager", "androidtv.basetv", "androidtv.androidtv", "androidtv.firetv"],
    install_requires=["adb-shell>=0.4.0", "pure-python-adb>=0.3.0.dev0"],
    extras_require={"async": ["aiofiles>=0.4.0", "async_timeout>=3.0.0"], "usb": ["adb-shell[usb]>=0.4.0"]},
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2",
    ],
    test_suite="tests",
)



"""Define patches used for androidtv tests."""

try:
    # Python3
    from unittest.mock import patch
except ImportError:
    # Python2
    from mock import patch


KEY_PYTHON = "python"
KEY_SERVER = "server"

ADB_DEVICE_TCP_FAKE = "AdbDeviceTcpFake"
CLIENT_FAKE_SUCCESS = "ClientFakeSuccess"
CLIENT_FAKE_FAIL = "ClientFakeFail"
DEVICE_FAKE = "DeviceFake"


class AdbDeviceTcpFake(object):
    """A fake of the `adb_shell.adb_device.AdbDeviceTcp` class."""

    def __init__(self, *args, **kwargs):
        """Initialize a fake `adb_shell.adb_device.AdbDeviceTcp` instance."""
        self.available = False

    def close(self):
        """Close the socket connection."""
        self.available = False

    def connect(self, *args, **kwargs):
        """Try to connect to a device."""
        raise NotImplementedError

    def push(self, *args, **kwargs):
        """Push a file to the device."""

    def pull(self, *args, **kwargs):
        """Pull a file from the device."""

    def shell(self, cmd, *args, **kwargs):
        """Send an ADB shell command."""
        return None


class ClientFakeSuccess(object):
    """A fake of the `ppadb.client.Client` class when the connection and shell commands succeed."""

    def __init__(self, host="127.0.0.1", port=5037):
        """Initialize a `ClientFakeSuccess` instance."""
        self._devices = []

    def device(self, serial):
        """Mock the `Client.device` method when the device is connected via ADB."""
        device = DeviceFake(serial)
        self._devices.append(device)
        return device


class ClientFakeFail(object):
    """A fake of the `ppadb.client.Client` class when the connection and shell commands fail."""

    def __init__(self, host="127.0.0.1", port=5037):
        """Initialize a `ClientFakeFail` instance."""
        self._devices = []

    def device(self, serial):
        """Mock the `Client.device` method when the device is not connected via ADB."""
        self._devices = []


class DeviceFake(object):
    """A fake of the `ppadb.device.Device` class."""

    def __init__(self, host):
        """Initialize a `DeviceFake` instance."""
        self.host = host

    def push(self, *args, **kwargs):
        """Push a file to the device."""

    def pull(self, *args, **kwargs):
        """Pull a file from the device."""

    def shell(self, cmd):
        """Send an ADB shell command."""
        raise NotImplementedError

    def screencap(self):
        """Take a screencap."""
        raise NotImplementedError


def patch_connect(success):
    """Mock the `adb_shell.adb_device.AdbDeviceTcp` and `ppadb.client.Client` classes."""

    def connect_success_python(self, *args, **kwargs):
        """Mock the `AdbDeviceTcpFake.connect` method when it succeeds."""
        self.available = True

    def connect_fail_python(self, *args, **kwargs):
        """Mock the `AdbDeviceTcpFake.connect` method when it fails."""
        raise OSError

    if success:
        return {
            KEY_PYTHON: patch("{}.{}.connect".format(__name__, ADB_DEVICE_TCP_FAKE), connect_success_python),
            KEY_SERVER: patch("androidtv.adb_manager.adb_manager_sync.Client", ClientFakeSuccess),
        }
    return {
        KEY_PYTHON: patch("{}.{}.connect".format(__name__, ADB_DEVICE_TCP_FAKE), connect_fail_python),
        KEY_SERVER: patch("androidtv.adb_manager.adb_manager_sync.Client", ClientFakeFail),
    }


def patch_shell(response=None, error=False):
    """Mock the `AdbDeviceTcpFake.shell` and `DeviceFake.shell` methods."""

    if isinstance(response, list):
        response_list = response
    else:
        response_list = [response]

    next_response = 0

    def shell_success(self, cmd, *args, **kwargs):
        """Mock the `AdbDeviceTcpFake.shell` and `DeviceFake.shell` methods when they are successful."""
        self.shell_cmd = cmd
        nonlocal next_response
        nonlocal response_list
        res = response_list[next_response]
        next_response = (next_response + 1) % len(response_list)
        return res

    def shell_fail_python(self, cmd, *args, **kwargs):
        """Mock the `AdbDeviceTcpFake.shell` method when it fails."""
        self.shell_cmd = cmd
        raise AttributeError

    def shell_fail_server(self, cmd):
        """Mock the `DeviceFake.shell` method when it fails."""
        self.shell_cmd = cmd
        raise ConnectionResetError

    if not error:
        return {
            KEY_PYTHON: patch("{}.{}.shell".format(__name__, ADB_DEVICE_TCP_FAKE), shell_success),
            KEY_SERVER: patch("{}.{}.shell".format(__name__, DEVICE_FAKE), shell_success),
        }
    return {
        KEY_PYTHON: patch("{}.{}.shell".format(__name__, ADB_DEVICE_TCP_FAKE), shell_fail_python),
        KEY_SERVER: patch("{}.{}.shell".format(__name__, DEVICE_FAKE), shell_fail_server),
    }


PATCH_PUSH = {
    KEY_PYTHON: patch("{}.{}.push".format(__name__, ADB_DEVICE_TCP_FAKE)),
    KEY_SERVER: patch("{}.{}.push".format(__name__, DEVICE_FAKE)),
}

PATCH_PULL = {
    KEY_PYTHON: patch("{}.{}.pull".format(__name__, ADB_DEVICE_TCP_FAKE)),
    KEY_SERVER: patch("{}.{}.pull".format(__name__, DEVICE_FAKE)),
}

PATCH_ADB_DEVICE_TCP = patch("androidtv.adb_manager.adb_manager_sync.AdbDeviceTcp", AdbDeviceTcpFake)

PATCH_ADB_DEVICE_USB = patch("androidtv.adb_manager.adb_manager_sync.AdbDeviceUsb", AdbDeviceTcpFake)

PATCH_ADB_SERVER_RUNTIME_ERROR = patch("{}.{}.device".format(__name__, CLIENT_FAKE_SUCCESS), side_effect=RuntimeError)


class CustomException(Exception):
    """A custom exception type."""


PATCH_CONNECT_FAIL_CUSTOM_EXCEPTION = {
    KEY_PYTHON: patch("{}.{}.connect".format(__name__, ADB_DEVICE_TCP_FAKE), side_effect=CustomException),
    KEY_SERVER: patch("{}.{}.device".format(__name__, CLIENT_FAKE_SUCCESS), side_effect=CustomException),
}


def patch_calls(obj, wraps):
    """Patch a method call without changing its behavior.

    Parameters
    ----------
    obj
        The object whose method will be patched (i.e., `self`)
    wraps
        The method that is being patched (i.e., `self.method`)

    Returns
    -------
    The patched method

    """
    return patch.object(type(obj), wraps.__name__.split()[-1], wraps=wraps)



"""Classes to manage ADB connections.

* :py:class:`ADBPythonSync` utilizes a Python implementation of the ADB protocol.
* :py:class:`ADBServerSync` utilizes an ADB server to communicate with the device.

"""


from contextlib import contextmanager
import logging
import sys
import threading

from adb_shell.adb_device import AdbDeviceTcp, AdbDeviceUsb
from adb_shell.auth.sign_pythonrsa import PythonRSASigner
from ppadb.client import Client

from ..constants import (
    DEFAULT_ADB_TIMEOUT_S,
    DEFAULT_AUTH_TIMEOUT_S,
    DEFAULT_LOCK_TIMEOUT_S,
    DEFAULT_TRANSPORT_TIMEOUT_S,
)
from ..exceptions import LockNotAcquiredException

_LOGGER = logging.getLogger(__name__)

#: Use a timeout for the ADB threading lock if it is supported
LOCK_KWARGS = {"timeout": DEFAULT_LOCK_TIMEOUT_S} if sys.version_info[0] > 2 and sys.version_info[1] > 1 else {}

if sys.version_info[0] == 2:  # pragma: no cover
    FileNotFoundError = IOError  # pylint: disable=redefined-builtin


@contextmanager
def _acquire(lock):
    """Handle acquisition and release of a ``threading.Lock`` object with ``LOCK_KWARGS`` keyword arguments.

    Parameters
    ----------
    lock : threading.Lock
        The lock that we will try to acquire

    Yields
    ------
    acquired : bool
        Whether or not the lock was acquired

    Raises
    ------
    LockNotAcquiredException
        Raised if the lock was not acquired

    """
    try:
        acquired = lock.acquire(**LOCK_KWARGS)
        if not acquired:
            raise LockNotAcquiredException
        yield acquired

    finally:
        if acquired:
            lock.release()


class ADBPythonSync(object):
    """A manager for ADB connections that uses a Python implementation of the ADB protocol.

    Parameters
    ----------
    host : str
        The address of the device; may be an IP address or a host name
    port : int
        The device port to which we are connecting (default is 5555)
    adbkey : str
        The path to the ``adbkey`` file for ADB authentication
    signer : PythonRSASigner, None
        The signer for the ADB keys, as loaded by :meth:`ADBPythonSync.load_adbkey`

    """

    def __init__(self, host, port, adbkey="", signer=None):
        self.host = host
        self.port = int(port)
        self.adbkey = adbkey

        if host:
            self._adb = AdbDeviceTcp(host=self.host, port=self.port, default_transport_timeout_s=DEFAULT_ADB_TIMEOUT_S)
        else:
            self._adb = AdbDeviceUsb(default_transport_timeout_s=DEFAULT_ADB_TIMEOUT_S)

        self._signer = signer

        # use a lock to make sure that ADB commands don't overlap
        self._adb_lock = threading.Lock()

    @property
    def available(self):
        """Check whether the ADB connection is intact.

        Returns
        -------
        bool
            Whether or not the ADB connection is intact

        """
        return self._adb.available

    def close(self):
        """Close the ADB socket connection."""
        self._adb.close()

    def connect(
        self,
        log_errors=True,
        auth_timeout_s=DEFAULT_AUTH_TIMEOUT_S,
        transport_timeout_s=DEFAULT_TRANSPORT_TIMEOUT_S,
    ):
        """Connect to an Android TV / Fire TV device.

        Parameters
        ----------
        log_errors : bool
            Whether errors should be logged
        auth_timeout_s : float
            Authentication timeout (in seconds)
        transport_timeout_s : float
            Transport timeout (in seconds)

        Returns
        -------
        bool
            Whether or not the connection was successfully established and the device is available

        """
        try:
            with _acquire(self._adb_lock):
                # Catch exceptions
                try:
                    # Connect with authentication
                    if self.adbkey:
                        if not self._signer:
                            self._signer = self.load_adbkey(self.adbkey)

                        self._adb.connect(
                            rsa_keys=[self._signer],
                            transport_timeout_s=transport_timeout_s,
                            auth_timeout_s=auth_timeout_s,
                        )

                    # Connect without authentication
                    else:
                        self._adb.connect(transport_timeout_s=transport_timeout_s, auth_timeout_s=auth_timeout_s)

                    # ADB connection successfully established
                    _LOGGER.debug("ADB connection to %s:%d successfully established", self.host, self.port)
                    return True

                except OSError as exc:
                    if log_errors:
                        if exc.strerror is None:
                            exc.strerror = "Timed out trying to connect to ADB device."
                        _LOGGER.warning(
                            "Couldn't connect to %s:%d.  %s: %s",
                            self.host,
                            self.port,
                            exc.__class__.__name__,
                            exc.strerror,
                        )

                    # ADB connection attempt failed
                    self.close()
                    return False

                except Exception as exc:  # pylint: disable=broad-except
                    if log_errors:
                        _LOGGER.warning(
                            "Couldn't connect to %s:%d.  %s: %s", self.host, self.port, exc.__class__.__name__, exc
                        )

                    # ADB connection attempt failed
                    self.close()
                    return False

        except LockNotAcquiredException:
            _LOGGER.warning("Couldn't connect to %s:%d because adb-shell lock not acquired.", self.host, self.port)
            self.close()
            return False

    @staticmethod
    def load_adbkey(adbkey):
        """Load the ADB keys.

        Parameters
        ----------
        adbkey : str
            The path to the ``adbkey`` file for ADB authentication

        Returns
        -------
        PythonRSASigner
            The ``PythonRSASigner`` with the key files loaded

        """
        # private key
        with open(adbkey) as f:
            priv = f.read()

        # public key
        try:
            with open(adbkey + ".pub") as f:
                pub = f.read()
        except FileNotFoundError:
            pub = ""

        return PythonRSASigner(pub, priv)

    def pull(self, local_path, device_path):
        """Pull a file from the device using the Python ADB implementation.

        Parameters
        ----------
        local_path : str
            The path where the file will be saved
        device_path : str
            The file on the device that will be pulled

        """
        if not self.available:
            _LOGGER.debug(
                "ADB command not sent to %s:%d because adb-shell connection is not established: pull(%s, %s)",
                self.host,
                self.port,
                local_path,
                device_path,
            )
            return

        with _acquire(self._adb_lock):
            _LOGGER.debug(
                "Sending command to %s:%d via adb-shell: pull(%s, %s)", self.host, self.port, local_path, device_path
            )
            self._adb.pull(device_path, local_path)
            return

    def push(self, local_path, device_path):
        """Push a file to the device using the Python ADB implementation.

        Parameters
        ----------
        local_path : str
            The file that will be pushed to the device
        device_path : str
            The path where the file will be saved on the device

        """
        if not self.available:
            _LOGGER.debug(
                "ADB command not sent to %s:%d because adb-shell connection is not established: push(%s, %s)",
                self.host,
                self.port,
                local_path,
                device_path,
            )
            return

        with _acquire(self._adb_lock):
            _LOGGER.debug(
                "Sending command to %s:%d via adb-shell: push(%s, %s)", self.host, self.port, local_path, device_path
            )
            self._adb.push(local_path, device_path)
            return

    def screencap(self):
        """Take a screenshot using the Python ADB implementation.

        Returns
        -------
        bytes
            The screencap as a binary .png image

        """
        if not self.available:
            _LOGGER.debug(
                "ADB screencap not taken from %s:%d because adb-shell connection is not established",
                self.host,
                self.port,
            )
            return None

        with _acquire(self._adb_lock):
            _LOGGER.debug("Taking screencap from %s:%d via adb-shell", self.host, self.port)
            result = self._adb.shell("screencap -p", decode=False)
            if result and result[5:6] == b"\r":
                return result.replace(b"\r\n", b"\n")
            return result

    def shell(self, cmd):
        """Send an ADB command using the Python ADB implementation.

        Parameters
        ----------
        cmd : str
            The ADB command to be sent

        Returns
        -------
        str, None
            The response from the device, if there is a response

        """
        if not self.available:
            _LOGGER.debug(
                "ADB command not sent to %s:%d because adb-shell connection is not established: %s",
                self.host,
                self.port,
                cmd,
            )
            return None

        with _acquire(self._adb_lock):
            _LOGGER.debug("Sending command to %s:%d via adb-shell: %s", self.host, self.port, cmd)
            return self._adb.shell(cmd)


class ADBServerSync(object):
    """A manager for ADB connections that uses an ADB server.

    Parameters
    ----------
    host : str
        The address of the device; may be an IP address or a host name
    port : int
        The device port to which we are connecting (default is 5555)
    adb_server_ip : str
        The IP address of the ADB server
    adb_server_port : int
        The port for the ADB server

    """

    def __init__(self, host, port=5555, adb_server_ip="", adb_server_port=5037):
        self.host = host
        self.port = int(port)
        self.adb_server_ip = adb_server_ip
        self.adb_server_port = adb_server_port
        self._adb_client = None
        self._adb_device = None

        # keep track of whether the ADB connection is intact
        self._available = False

        # use a lock to make sure that ADB commands don't overlap
        self._adb_lock = threading.Lock()

    @property
    def available(self):
        """Check whether the ADB connection is intact.

        Returns
        -------
        bool
            Whether or not the ADB connection is intact

        """
        if not self._adb_client or not self._adb_device:
            return False

        return self._available

    def close(self):
        """Close the ADB server socket connection.

        Currently, this doesn't do anything except set ``self._available = False``.

        """
        self._available = False

    def connect(self, log_errors=True):
        """Connect to an Android TV / Fire TV device.

        Parameters
        ----------
        log_errors : bool
            Whether errors should be logged

        Returns
        -------
        bool
            Whether or not the connection was successfully established and the device is available

        """
        try:
            with _acquire(self._adb_lock):
                # Catch exceptions
                try:
                    self._adb_client = Client(host=self.adb_server_ip, port=self.adb_server_port)
                    self._adb_device = self._adb_client.device("{}:{}".format(self.host, self.port))

                    # ADB connection successfully established
                    if self._adb_device:
                        _LOGGER.debug(
                            "ADB connection to %s:%d via ADB server %s:%d successfully established",
                            self.host,
                            self.port,
                            self.adb_server_ip,
                            self.adb_server_port,
                        )
                        self._available = True
                        return True

                    # ADB connection attempt failed (without an exception)
                    if log_errors:
                        _LOGGER.warning(
                            "Couldn't connect to %s:%d via ADB server %s:%d because the server is not connected to the device",
                            self.host,
                            self.port,
                            self.adb_server_ip,
                            self.adb_server_port,
                        )

                    self.close()
                    self._available = False
                    return False

                # ADB connection attempt failed
                except Exception as exc:  # noqa pylint: disable=broad-except
                    if log_errors:
                        _LOGGER.warning(
                            "Couldn't connect to %s:%d via ADB server %s:%d, error: %s",
                            self.host,
                            self.port,
                            self.adb_server_ip,
                            self.adb_server_port,
                            exc,
                        )

                    self.close()
                    self._available = False
                    return False

        except LockNotAcquiredException:
            _LOGGER.warning(
                "Couldn't connect to %s:%d via ADB server %s:%d because pure-python-adb lock not acquired.",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
            )
            self.close()
            self._available = False
            return False

    def pull(self, local_path, device_path):
        """Pull a file from the device using an ADB server.

        Parameters
        ----------
        local_path : str
            The path where the file will be saved
        device_path : str
            The file on the device that will be pulled

        """
        if not self.available:
            _LOGGER.debug(
                "ADB command not sent to %s:%d via ADB server %s:%d because pure-python-adb connection is not established: pull(%s, %s)",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
                local_path,
                device_path,
            )
            return

        with _acquire(self._adb_lock):
            _LOGGER.debug(
                "Sending command to %s:%d via ADB server %s:%d: pull(%s, %s)",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
                local_path,
                device_path,
            )
            self._adb_device.pull(device_path, local_path)
            return

    def push(self, local_path, device_path):
        """Push a file to the device using an ADB server.

        Parameters
        ----------
        local_path : str
            The file that will be pushed to the device
        device_path : str
            The path where the file will be saved on the device

        """
        if not self.available:
            _LOGGER.debug(
                "ADB command not sent to %s:%d via ADB server %s:%d because pure-python-adb connection is not established: push(%s, %s)",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
                local_path,
                device_path,
            )
            return

        with _acquire(self._adb_lock):
            _LOGGER.debug(
                "Sending command to %s:%d via ADB server %s:%d: push(%s, %s)",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
                local_path,
                device_path,
            )
            self._adb_device.push(local_path, device_path)
            return

    def screencap(self):
        """Take a screenshot using an ADB server.

        Returns
        -------
        bytes, None
            The screencap as a binary .png image, or ``None`` if there was an ``IndexError`` exception

        """
        if not self.available:
            _LOGGER.debug(
                "ADB screencap not taken from %s:%d via ADB server %s:%d because pure-python-adb connection is not established",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
            )
            return None

        with _acquire(self._adb_lock):
            _LOGGER.debug(
                "Taking screencap from %s:%d via ADB server %s:%d",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
            )
            return self._adb_device.screencap()

    def shell(self, cmd):
        """Send an ADB command using an ADB server.

        Parameters
        ----------
        cmd : str
            The ADB command to be sent

        Returns
        -------
        str, None
            The response from the device, if there is a response

        """
        if not self.available:
            _LOGGER.debug(
                "ADB command not sent to %s:%d via ADB server %s:%d because pure-python-adb connection is not established: %s",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
                cmd,
            )
            return None

        with _acquire(self._adb_lock):
            _LOGGER.debug(
                "Sending command to %s:%d via ADB server %s:%d: %s",
                self.host,
                self.port,
                self.adb_server_ip,
                self.adb_server_port,
                cmd,
            )
            return self._adb_device.shell(cmd)



"""Communicate with an Amazon Fire TV device via ADB over a network.

ADB Debugging must be enabled.
"""


import logging

from .base_firetv import BaseFireTV
from ..basetv.basetv_sync import BaseTVSync

_LOGGER = logging.getLogger(__name__)


class FireTVSync(BaseTVSync, BaseFireTV):
    """Representation of an Amazon Fire TV device.

    Parameters
    ----------
    host : str
        The address of the device; may be an IP address or a host name
    port : int
        The device port to which we are connecting (default is 5555)
    adbkey : str
        The path to the ``adbkey`` file for ADB authentication
    adb_server_ip : str
        The IP address of the ADB server
    adb_server_port : int
        The port for the ADB server
    state_detection_rules : dict, None
        A dictionary of rules for determining the state (see :class:`~androidtv.basetv.basetv.BaseTV`)
    signer : PythonRSASigner, None
        The signer for the ADB keys, as loaded by :meth:`androidtv.adb_manager.adb_manager_sync.ADBPythonSync.load_adbkey`

    """

    def __init__(
        self,
        host,
        port=5555,
        adbkey="",
        adb_server_ip="",
        adb_server_port=5037,
        state_detection_rules=None,
        signer=None,
    ):  # pylint: disable=super-init-not-called
        BaseTVSync.__init__(self, host, port, adbkey, adb_server_ip, adb_server_port, state_detection_rules, signer)

    @classmethod
    def from_base(cls, base_tv):
        """Construct a `FireTVSync` object from a `BaseTVSync` object.

        Parameters
        ----------
        base_tv : BaseTVSync
            The object that will be converted to a `FireTVSync` object

        Returns
        -------
        ftv : FireTVSync
            The constructed `FireTVSync` object

        """
        # pylint: disable=protected-access
        ftv = cls(
            base_tv.host,
            base_tv.port,
            base_tv.adbkey,
            base_tv.adb_server_ip,
            base_tv.adb_server_port,
            base_tv._state_detection_rules,
        )
        ftv._adb = base_tv._adb
        ftv.device_properties = base_tv.device_properties
        ftv.installed_apps = base_tv.installed_apps
        ftv.max_volume = base_tv.max_volume
        return ftv

    # ======================================================================= #
    #                                                                         #
    #                          Home Assistant Update                          #
    #                                                                         #
    # ======================================================================= #
    def update(self, get_running_apps=True, lazy=True):
        """Get the info needed for a Home Assistant update.

        Parameters
        ----------
        get_running_apps : bool
            Whether or not to get the :meth:`~androidtv.firetv.firetv_sync.FireTVSync.running_apps` property
        lazy : bool
            Whether or not to continue retrieving properties if the device is off or the screensaver is running

        Returns
        -------
        state : str
            The state of the device
        current_app : str
            The current running app
        running_apps : list
            A list of the running apps if ``get_running_apps`` is True, otherwise the list ``[current_app]``
        hdmi_input : str, None
            The HDMI input, or ``None`` if it could not be determined

        """
        # Get the properties needed for the update
        (
            screen_on,
            awake,
            wake_lock_size,
            current_app,
            media_session_state,
            running_apps,
            hdmi_input,
        ) = self.get_properties(get_running_apps=get_running_apps, lazy=lazy)

        return self._update(
            screen_on, awake, wake_lock_size, current_app, media_session_state, running_apps, hdmi_input
        )

    # ======================================================================= #
    #                                                                         #
    #                               Properties                                #
    #                                                                         #
    # ======================================================================= #
    def get_properties(self, get_running_apps=True, lazy=False):
        """Get the properties needed for Home Assistant updates.

        Parameters
        ----------
        get_running_apps : bool
            Whether or not to get the :meth:`~androidtv.firetv.firetv_sync.FireTVSync.running_apps` property
        lazy : bool
            Whether or not to continue retrieving properties if the device is off or the screensaver is running

        Returns
        -------
        screen_on : bool, None
            Whether or not the device is on, or ``None`` if it was not determined
        awake : bool, None
            Whether or not the device is awake (screensaver is not running), or ``None`` if it was not determined
        wake_lock_size : int, None
            The size of the current wake lock, or ``None`` if it was not determined
        current_app : str, None
            The current app property, or ``None`` if it was not determined
        media_session_state : int, None
            The state from the output of ``dumpsys media_session``, or ``None`` if it was not determined
        running_apps : list, None
            A list of the running apps, or ``None`` if it was not determined
        hdmi_input : str, None
            The HDMI input, or ``None`` if it could not be determined

        """
        screen_on, awake, wake_lock_size = self.screen_on_awake_wake_lock_size()
        if lazy and not (screen_on and awake):
            return screen_on, awake, wake_lock_size, None, None, None, None

        current_app, media_session_state = self.current_app_media_session_state()

        if get_running_apps:
            running_apps = self.running_apps()
        else:
            running_apps = None

        hdmi_input = self.get_hdmi_input()

        return screen_on, awake, wake_lock_size, current_app, media_session_state, running_apps, hdmi_input

    def get_properties_dict(self, get_running_apps=True, lazy=True):
        """Get the properties needed for Home Assistant updates and return them as a dictionary.

        Parameters
        ----------
        get_running_apps : bool
            Whether or not to get the :meth:`~androidtv.firetv.firetv_sync.FireTVSync.running_apps` property
        lazy : bool
            Whether or not to continue retrieving properties if the device is off or the screensaver is running

        Returns
        -------
        dict
             A dictionary with keys ``'screen_on'``, ``'awake'``, ``'wake_lock_size'``, ``'current_app'``,
             ``'media_session_state'``, ``'running_apps'``, and ``'hdmi_input'``

        """
        (
            screen_on,
            awake,
            wake_lock_size,
            current_app,
            media_session_state,
            running_apps,
            hdmi_input,
        ) = self.get_properties(get_running_apps=get_running_apps, lazy=lazy)

        return {
            "screen_on": screen_on,
            "awake": awake,
            "wake_lock_size": wake_lock_size,
            "current_app": current_app,
            "media_session_state": media_session_state,
            "running_apps": running_apps,
            "hdmi_input": hdmi_input,
        }



"""Connect to a device and determine whether it's an Android TV or an Amazon Fire TV.

ADB Debugging must be enabled.
"""

from .androidtv.androidtv_sync import AndroidTVSync
from .basetv.basetv import state_detection_rules_validator
from .basetv.basetv_sync import BaseTVSync
from .constants import DEFAULT_AUTH_TIMEOUT_S, DEFAULT_TRANSPORT_TIMEOUT_S
from .firetv.firetv_sync import FireTVSync


__version__ = "0.0.73"


def setup(
    host,
    port=5555,
    adbkey="",
    adb_server_ip="",
    adb_server_port=5037,
    state_detection_rules=None,
    device_class="auto",
    auth_timeout_s=DEFAULT_AUTH_TIMEOUT_S,
    signer=None,
    transport_timeout_s=DEFAULT_TRANSPORT_TIMEOUT_S,
    log_errors=True,
):
    """Connect to a device and determine whether it's an Android TV or an Amazon Fire TV.

    Parameters
    ----------
    host : str
        The address of the device; may be an IP address or a host name
    port : int
        The device port to which we are connecting (default is 5555)
    adbkey : str
        The path to the ``adbkey`` file for ADB authentication
    adb_server_ip : str
        The IP address of the ADB server
    adb_server_port : int
        The port for the ADB server
    state_detection_rules : dict, None
        A dictionary of rules for determining the state (see :class:`~androidtv.basetv.basetv.BaseTV`)
    device_class : str
        The type of device: ``'auto'`` (detect whether it is an Android TV or Fire TV device), ``'androidtv'``, or ``'firetv'```
    auth_timeout_s : float
        Authentication timeout (in seconds)
    signer : PythonRSASigner, None
        The signer for the ADB keys, as loaded by :meth:`androidtv.adb_manager.adb_manager_sync.ADBPythonSync.load_adbkey`
    transport_timeout_s : float
        Transport timeout (in seconds)
    log_errors: bool
        Whether connection errors should be logged

    Returns
    -------
    AndroidTVSync, FireTVSync
        The representation of the device

    """
    if device_class == "androidtv":
        atv = AndroidTVSync(host, port, adbkey, adb_server_ip, adb_server_port, state_detection_rules, signer)
        atv.adb_connect(log_errors=log_errors, auth_timeout_s=auth_timeout_s, transport_timeout_s=transport_timeout_s)
        atv.get_device_properties()
        atv.get_installed_apps()
        return atv

    if device_class == "firetv":
        ftv = FireTVSync(host, port, adbkey, adb_server_ip, adb_server_port, state_detection_rules, signer)
        ftv.adb_connect(log_errors=log_errors, auth_timeout_s=auth_timeout_s, transport_timeout_s=transport_timeout_s)
        ftv.get_device_properties()
        ftv.get_installed_apps()
        return ftv

    if device_class != "auto":
        raise ValueError("`device_class` must be 'androidtv', 'firetv', or 'auto'.")

    aftv = BaseTVSync(host, port, adbkey, adb_server_ip, adb_server_port, state_detection_rules, signer)

    # establish the ADB connection
    aftv.adb_connect(log_errors=log_errors, auth_timeout_s=auth_timeout_s, transport_timeout_s=transport_timeout_s)

    # get device properties
    aftv.device_properties = aftv.get_device_properties()

    # get the installed apps
    aftv.get_installed_apps()

    # Fire TV
    if aftv.device_properties.get("manufacturer") == "Amazon":
        return FireTVSync.from_base(aftv)

    # Android TV
    return AndroidTVSync.from_base(aftv)


def ha_state_detection_rules_validator(exc):
    """Validate the rules (i.e., the ``state_detection_rules`` value) for a given app ID (i.e., a key in ``state_detection_rules``).

    See :class:`~androidtv.basetv.basetv.BaseTV` for more info about the ``state_detection_rules`` parameter.

    Parameters
    ----------
    exc : Exception
        The exception that will be raised if a rule is invalid

    Returns
    -------
    wrapped_state_detection_rules_validator : function
        A function that is the same as :func:`~androidtv.basetv.state_detection_rules_validator`, but with the ``exc`` argument provided

    """

    def wrapped_state_detection_rules_validator(rules):
        """Run :func:`~androidtv.basetv.state_detection_rules_validator` using the ``exc`` parameter from the parent function."""
        return state_detection_rules_validator(rules, exc)

    return wrapped_state_detection_rules_validator
